;===============================================================================
;   Fermentation temperature controller unit for home brewing using 16F648A
;-------------------------------------------------------------------------------

;   Tate Peñaranda - tate@xenir.com.ar
;   skypeid: srgokzilla - mobile: +54 9 341 5851961
;   Rosario, Argentina

;===============================================================================

;   This program is free software: you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation, either version 3 of the License, or
;   (at your option) any later version.

;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.

;   You should have received a copy of the GNU General Public License
;   along with this program.  If not, see <http://www.gnu.org/licenses/>.

;===============================================================================

;   The main purpose of this program (and circuit) is get a robust, reliable,
;   simple and cheap fermentation controller unit.
;   This was achieved using a well know microchip PIC, cheap and easy to get
;   in almost any country.
;   The operation mode it's pretty clear; you set (using buttons and display)
;   each temperature according to each fermentation stage and how many days the
;   system must remain using that temperature, turn on and the circuit will take
;   care of your beer temp. It supports up to 3 fermentation stages.
;   Of course this circuit did not chill the beer by itself, needs to be used in
;   conjunction with fridge, fermentation chamber or also electric heaters
;   at cold places.
;   The circuit was tested on many differents situations, and we can say that
;   still is performing very well since two years after his installation.
;   To check some photos <http://xenir.com.ar>
;
;   Some routines (LCD and 1Wire) are just adaptions from originals found on
;   'Microcontrolador PIC16F84 - Desarrollo de proyectos' written by
;   Enrique Palacios, Fernando Ramiro and Lucas López
;   Delays (retardos) routines were calculated using PicLoops 2.2
;
;   Quick PIC pinout:
;           1. LCD Data Bit 1 of 4
;           2. LCD Data Bit 2 of 4
;           3. DS18B20 Temp Sensor 1Wire data
;           4. MCLR (must be set to HI if MCLR is enabled)
;           5. GND (PIC power)
;           6. LCD Backlight control (through BC548)
;           7. Cooler/Heater output, to toggle relay or similar
;           8. Buzzer output
;           9. Clock for 7Segs display output (using CD4094)
;           10. 'Bajar' button input
;           11. 'Subir' button input
;           12. Xtal input 32.768 kHz
;           13. Xtal input 32.768 kHz
;           14. +5v (PIC power)
;           15. LCD Data Bit 3 of 4
;           16. LCD Data Bit 4 of 4
;           17. LCD R/S Pin / Data for 7Segs display output (using CD4094)
;           18. LCD Enable Pin

;------------------------------------------------------------------------------
;	Work to do...
;------------------------------------------------------------------------------
; - Check CRC of temperature reads.
; - Translate displayed messages to english
; - Translate spanish named variables to english

;===============================================================================
;   Begin program...
;===============================================================================

    LIST	   P=16F648A
	__CONFIG   _CP_OFF &  _WDT_ON & _PWRTE_ON & _INTOSC_OSC_NOCLKOUT & _BODEN_ON & _LVP_OFF & _MCLRE_OFF
	INCLUDE  <P16F648A.INC>

;-------------------------------------------------------------------------------
;	Define some simple macros to change banks
;-------------------------------------------------------------------------------
Bank0		MACRO
		bcf	STATUS,RP0
		bcf	STATUS,RP1
		ENDM

Bank1		MACRO
		bsf	STATUS,RP0
		bcf	STATUS,RP1
		ENDM

Bank2		MACRO
		bcf	STATUS,RP0
		bsf	STATUS,RP1
		ENDM

Bank3		MACRO
		bsf	STATUS,RP0
		bsf	STATUS,RP1
		ENDM

;-------------------------------------------------------------------------------
;	Define variables and his positions in PIC memory
;   (I prefer avoid using 'CBLOCK' and define positions manually).
;   'EEPROM => 0x??' means that current variable will be stored on position 0x??
;   in EEPROM memory at certain moment. Remember EEPROM memory store his values
;   even after poweroff.
;-------------------------------------------------------------------------------

Flags                           equ 0x20    ;EEPROM => 0x00
Flags1                          equ 0x21    ;EEPROM => 0x01

CounterA                        equ 0x22
CounterB                        equ 0x23
CounterC                        equ 0x24

LCD_ValorCaracter               equ 0x25
LCD_ApuntaCaracter              equ 0x26

OneWireData                       equ 0x27
OneWireBitCount                   equ 0x28

ProcesarTempTemporario          equ 0x29
ModoDeFuncionamientoTemporario  equ 0x2A
LCDTemporario                   equ 0x2B
SumaDiasTemporario              equ 0x2C

Unidades                        equ 0x2D
Decenas                         equ 0x2E

Botones                         equ 0x2F

Dias1raF                        equ	0x30    ;EEPROM => 0x02
Dias2daF                        equ	0x31    ;EEPROM => 0x03
DiasMadur                       equ	0x32    ;EEPROM => 0x04

TemperaturaLeidaLSB             equ 0x33
TemperaturaLeidaMSB             equ 0x34

LCDData                         equ 0x35

Texto1                          equ 0x36
Texto2                          equ 0x37
Texto3                          equ 0x38

Contador4SegundosParaDias       equ 0x39    ;EEPROM => 0x05
Contador800Segundos             equ 0x3A    ;EEPROM => 0x06
ContadorDias                    equ 0x3B    ;EEPROM => 0x07

Contador4Segundos               equ 0x3C
Contador4SegundosParaMinutos    equ 0x3D
SumaTotalMinutos                equ 0x3E

ModoDeFuncionamiento            equ 0x3F    ;EEPROM => 0x08

SumaDiasFases                   equ 0x40

Centenas                        equ 0x41

ContadorMinutos                 equ 0x42    ;EEPROM => 0x19

ContadorParaApagarBacklight     equ 0x43

NumeroAEscribir7S               equ 0x44
CantPulsos7S                    equ 0x45

CRC                             equ 0x46

CompararTemporario              equ 0x47
AComparar_1LSB                  equ 0x48
AComparar_1MSB                  equ 0x49
AComparar_2LSB                  equ 0x4A
AComparar_2MSB                  equ 0x4B

TemperaturaEntero               equ 0x4C
TemperaturaDecimal              equ 0x4D

TemperaturaAEscribirEntero      equ 0x4E
TemperaturaAEscribirDecimal     equ 0x4F

TemperaturaBuscadaMSB           equ 0x50
TemperaturaBuscadaLSB           equ 0x51

DeltaTLSB                       equ 0x52    ;EEPROM => 0x09
DeltaTMSB                       equ 0x53    ;EEPROM => 0x0A

TemperaturaBuscadaSuperiorLSB   equ 0x54
TemperaturaBuscadaSuperiorMSB   equ 0x55

TemperaturaBuscadaInferiorLSB   equ 0x56
TemperaturaBuscadaInferiorMSB   equ 0x57

TempModoTermostatoLSB           equ 0x58    ;EEPROM => 0x0B
TempModoTermostatoMSB           equ 0x59    ;EEPROM => 0x0C

TemporarioLSB                   equ 0x5A    ; Esto sólo se puede cambiar en una interrupción
TemporarioMSB                   equ 0x5B    ; Esto sólo se puede cambiar en una interrupción

Temp1raFLSB                     equ 0x5C    ;EEPROM => 0x0D
Temp1raFMSB                     equ 0x5D    ;EEPROM => 0x0E

Temp2daFLSB                     equ 0x5E    ;EEPROM => 0x0F
Temp2daFMSB                     equ 0x5F    ;EEPROM => 0x10

TempMaduracionLSB               equ 0x60    ;EEPROM => 0x11
TempMaduracionMSB               equ 0x61    ;EEPROM => 0x12

TempPasarAModoTermostatoLSB     equ 0x62    ;EEPROM => 0x13
TempPasarAModoTermostatoMSB     equ 0x63    ;EEPROM => 0x14

CantidadDeEscalones             equ 0x64    ;EEPROM => 0x15

MinutosE4                       equ 0x65    ;EEPROM => 0x16

TempE4LSB                       equ	0x66    ;EEPROM => 0x17
TempE4MSB                       equ 0x67    ;EEPROM => 0x18

CantidadDeEscalonesTemporario   equ 0x68
EscalonAjustando                equ 0x69

FlagTemp                        equ 0x6A

CuatroSegundosEncendido         equ 0x6B
MinutosEncendido                equ	0x6C    ;EEPROM => 0x20
HorasEncendido                  equ	0x6D    ;EEPROM => 0x21
DiasEncendido                   equ 0x6E    ;EEPROM => 0x22

PCL_Temp                        equ 0x7C
W_Temp                          equ 0x7D
STATUS_Temp                     equ 0x7E
PCLATH_Temp                     equ 0x7F

#DEFINE	ModoTermostato                      ModoDeFuncionamiento,0
#DEFINE	ModoFermentar                       ModoDeFuncionamiento,1
#DEFINE	ModoApagado                         ModoDeFuncionamiento,2

#DEFINE	LaTempEsNegativa                    Flags,0
#DEFINE	ModoSensorTipoB                     Flags,1
#DEFINE	Fermentando1                        Flags,2
#DEFINE	Fermentando2                        Flags,3
#DEFINE	Madurando                           Flags,4
#DEFINE	ProcesoFinalizado                   Flags,5
#DEFINE	LuegoTermostato                     Flags,6
#DEFINE	SeDeseaElBacklightPrendido          Flags,7

#DEFINE	ControlDeMacerado                   Flags1,0
#DEFINE	EstaEnElEscalon4                    Flags1,4	; Se podrá usar ,1?
#DEFINE	SalidaParaCalentar                  Flags1,5

#DEFINE	LCDDataPort                         PORTA
#DEFINE	LCDRSPin                            PORTA,0
#DEFINE	SieteSeg_D                          PORTA,0
#DEFINE	LCDEPin                             PORTA,1
#DEFINE	Bus1Linea                           PORTA,4
#DEFINE	BotonSubir                          PORTB,5
#DEFINE	BotonBajar                          PORTB,4
#DEFINE	Backlight                           PORTB,0
#DEFINE	ActivarSalida                       PORTB,1

#DEFINE	Buzzer                              PORTB,2
#DEFINE	SieteSeg_CLK                        PORTB,3

#DEFINE	DS1820_Skip_ROM_Command             0xCC
#DEFINE	DS1820_Convert_T_Command            0x44
#DEFINE	DS1820_Read_Scratchpad_Command      0xBE
#DEFINE	DS1820_Write_Scratchpad_Command     0x4E

org 0x00 			; Acá cae cuando se inicia el PIC (o se resetea)
	goto	Inicio
	
org 0x04 			; Acá cuando acontece alguna interrupción
	movwf	W_Temp
	
	swapf	STATUS,W
	Bank0
	movwf	STATUS_Temp
	
	movf	PCLATH,W
	movwf	PCLATH_Temp
	
;	movf	PCL,W
;	movwf	PCL_Temp
	
	bcf	PCLATH,3
	goto	Interrupcion

Mensajes
	addwf	PCL,F
Modo
	DT "Modo", 0x00
Termostato
	DT "Termostato", 0x00
Fermentar
	DT "Fermentar", 0x00
Configurar
	DT .143, "Configurar", 0x00    ; 143 = '¿'
	;DT b'10011111', "Configurar", 0x00
Programa
	DT "programa?", 0x00
Grados
	DT .2, "C", 0x00    ; 2 = 'º'
GradosAlReves
	DT "C", .2, 0x00    ; A veces escribimos el display de atrás para adelante,
	                    ; para que quede todo perfectamente centrado y no escribir
	                    ; de más al pedo.
Iniciar
	DT .143, "Iniciar", 0x00
	;DT b'10011111', "Iniciar", 0x00
Detener
	DT .143, "Detener", 0x00
	;DT b'10011111', "Detener", 0x00
Nada
	DT 0x00
Apagado
	DT "Apagado", 0x00
Si
	DT "SI", 0x00
No
	DT "NO", 0x00
DespuesTermostato
	DT .143, "Luego Termost.?", 0x00
	;DT b'10011111', "Luego enfriar?", 0x00
Guardar
	DT "Guardar", 0x00
LosAjustes
	DT "ajustes", 0x00
Primera
	DT "1ra", 0x00
Segunda
	DT "2da", 0x00
Fermentacion
	DT "fermentaci", .141, "n", 0x00    ;.141 = 'í'
	;DT "fermentaci", b'10100010', "n", 0x00
Maduracion
	DT "Maduraci", .141, "n", 0x00
	;DT "Maduraci", b'10100010', "n", 0x00
Dias
	DT "D", .163, "as", 0x00    ; .163 = 'í'
	;DT "D", b'10100001', "as", 0x00
FinDelProceso
	DT " Fin del proceso", 0x00
TError
	DT "Error", 0x00
Restante
	DT "Restante", 0x00
Fase
	DT "fase", 0x00
Total
	DT "total", 0x00
TempPunto
	DT "Temp.", 0x00
Tolerancia
	DT "Tolerancia:", 0x00
Cancelar
	DT "Cancelar", 0x00
Macerar
	DT "Mac.", 0x00

org 0x100
ConvertirA7S
	addwf	PCL,F
        retlw   b'11111100'	; 0
        retlw   b'01100000'	; 1
        retlw   b'11011010'	; 2
        retlw   b'11110010'	; 3
        retlw   b'01100110'	; 4
        retlw   b'10110110'	; 5
        retlw   b'10111110'	; 6
        retlw   b'11100000'	; 7
        retlw   b'11111110'	; 8
        retlw   b'11110110'	; 9
        retlw   b'00000010'	; -
        retlw   b'00000000'	; Nada
TablaCalcularElDecimal
        addwf	PCL,F
        retlw   .0	;0
	retlw   .1	;0.0625
	retlw   .1	;0.125
	retlw   .2	;0.1875
	retlw   .3	;0.25
	retlw   .3	;0.3125
	retlw   .4	;0.375
	retlw   .4	;0.4375
	retlw   .5	;0.5
	retlw   .6	;0.5625
	retlw   .6	;0.625
	retlw   .7	;0.6875
	retlw   .8	;0.75
	retlw   .8	;0.8125
	retlw   .9	;0.875
	retlw   .9	;0.9375
	retlw   .0	;1		Supuestamente acá no entraría nunca, pero por las dudas lo dejamos.
TablaCalcularPrimerDecimal
        addwf	PCL,F
        retlw   .0	;0
	retlw   .0	;0.0625
	retlw   .1	;0.125
	retlw   .1	;0.1875
	retlw   .2	;0.25
	retlw   .3	;0.3125
	retlw   .3	;0.375
	retlw   .4	;0.4375
	retlw   .5	;0.5
	retlw   .5	;0.5625
	retlw   .6	;0.625
	retlw   .6	;0.6875
	retlw   .7	;0.75
	retlw   .8	;0.8125
	retlw   .8	;0.875
	retlw   .9	;0.9375
	retlw   .0	;1
TablaCalcularSegundoDecimal
        addwf	PCL,F
        retlw   .0	;0
	retlw   .6	;0.0625
	retlw   .3	;0.125
	retlw   .9	;0.1875
	retlw   .5	;0.25
	retlw   .1	;0.3125
	retlw   .8	;0.375
	retlw   .4	;0.4375
	retlw   .0	;0.5
	retlw   .6	;0.5625
	retlw   .3	;0.625
	retlw   .9	;0.6875
	retlw   .5	;0.75
	retlw   .1	;0.8125
	retlw   .8	;0.875
	retlw   .4	;0.9375
	retlw   .0	;1

CantidadDe
	DT "Cantidad de", 0x00
Escalones
	DT "escalones:", 0x00
Escalon
	DT "escal", .141, "n", 0x00
	;DT "escal", b'10100010', "n", 0x00
Esc
	DT "esc.", 0x00
Mins
	DT "Mins.", 0x00
Snim
	DT "snim ", 0x00
T1er
	DT "1er", 0x00
T2do
	DT "2do", 0x00
T3er
	DT "3er", 0x00
T4to
	DT "4to", 0x00
Macerado
	DT "Macerado?", 0x00
Ed
	DT " ed ", 0x00
Transcurrido
	DT "T.", 0x00		;DT "Transcurrido", 0x00
Reset
	DT "Reset?", 0x00
Firmware
	DT "FW V1.3.1", 0x00
SalidaPara
	DT "Salida para:", 0x00
Enfriar
	DT "Enfriar", 0x00
Calentar
	DT "Calentar", 0x00
TiempoEncendido
	DT "Tiempo activado", 0x00
FermentacionMaduracion
	DT "Ferment / Madur?", 0x00

Interrupcion
	;bcf	PCLATH,3	; Ya viene limpio este bit
	
	call	RevisarBotones
	
	movf	Botones,0
	btfss	STATUS,Z
	call	SeAccionoUnBoton
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	Info0
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	PreguntaConfigurarPrograma
	
	movf	Botones,0
	xorlw	b'00000011'
	btfsc	STATUS,Z
	call	CambiarBacklight
	
	goto	FinInt

Info0
	bsf	Backlight
	clrf	ContadorParaApagarBacklight
	
	bsf	PCLATH,3
	
;	btfsc	ControlDeMacerado
;	goto	Info0Macerado
	
	bcf	PCLATH,3
	
	btfss	ModoFermentar			;Si no está fermentando no lo muestra
	goto	Info2
	
	movlw	Fase
	movwf	Texto1
	
	movf	ContadorDias,0
	subwf	SumaDiasFases,0
	
	movwf	Texto2
	
	call	DibujarMenuInfo
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	Info1
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	PreguntaConfigurarPrograma
	goto	FinInt

Info1
	btfss	ModoFermentar			;Si no está fermentando no lo muestra
	goto	PreguntaConfigurarPrograma
	
	movlw	Total
	movwf	Texto1
	
	movf	Dias1raF,W
	addwf	Dias2daF,W
	addwf	DiasMadur,W
	movwf	SumaDiasTemporario
	
	movf	ContadorDias,W
	subwf	SumaDiasTemporario,W
	
	movwf	Texto2
	
	call	DibujarMenuInfo
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	Info2
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	Info0
	goto	FinInt

Info2
	call	LCD_Borra
	
	movlw	TiempoEncendido
	call	LCD_MensajeAlto
	
	call	LCD_Linea2
	
	movf	DiasEncendido,W
	call	EscribirEntero
	
	movlw	.3
	call	LCD_PosicionLinea2
	
	movlw	'D'
	call	LCD_EnviaDato
	movlw	' '
	call	LCD_EnviaDato
	movlw	' '
	call	LCD_EnviaDato
	
	movf	HorasEncendido,W
	call	EscribirEntero
	
	movlw	' '
	call	LCD_EnviaDato
	movlw	'h'
	call	LCD_EnviaDato
	movlw	' '
	call	LCD_EnviaDato
	movlw	' '
	call	LCD_EnviaDato
	
	movf	MinutosEncendido,W
	call	EscribirEntero
	
	movlw	' '
	call	LCD_EnviaDato
	movlw	'm'
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	ModoFuncionamiento
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	Info1
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinInt
	
	clrf	MinutosEncendido
	clrf	HorasEncendido
	clrf	DiasEncendido
	
	Bank1
	movlw	0x20
	movwf	EEADR
	bcf	STATUS,RP0
	movf	MinutosEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x21
	movwf	EEADR
	bcf	STATUS,RP0
	movf	HorasEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x22
	movwf	EEADR
	bcf	STATUS,RP0
	movf	DiasEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	goto	Info2

ModoFuncionamiento
	call	LCD_Borra
	
	movlw	Modo
	call	LCD_Mensaje
	
	movlw	':'
	call	LCD_EnviaDato
	
	call	LCD_Linea2
	
	movlw	TError
	
	btfsc	ModoApagado
	movlw	Apagado
	
	btfsc	ModoTermostato
	movlw	Termostato
	
	btfsc	ModoFermentar
	movlw	Fermentar
	
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	SeleccionarSalida	;AjustarTempTermostato
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	Info2
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinInt
	
	movf	ModoDeFuncionamiento,W
	movwf	ModoDeFuncionamientoTemporario
	
CambiarModoDeFuncionamiento
	call	LCD_Borra
	
	movlw	Modo
	call	LCD_Mensaje
	
	movlw	':'
	call	LCD_EnviaDato
	
	call	LCD_Linea2
	
	movlw	TError
	
	btfsc	ModoDeFuncionamientoTemporario,0
	movlw	Termostato
	
	btfsc	ModoDeFuncionamientoTemporario,1
	movlw	Fermentar
	
	btfsc	ModoDeFuncionamientoTemporario,2
	movlw	Apagado
	
	call	LCD_Mensaje
	
	movlw	.1
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	goto	ModoFuncionamiento
	
	bcf	STATUS,C
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	rrf	ModoDeFuncionamientoTemporario,F
	
	btfsc	STATUS,C
	bsf	ModoDeFuncionamientoTemporario,2
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	rlf	ModoDeFuncionamientoTemporario,F
	
	btfsc	ModoDeFuncionamientoTemporario,3
	bsf	ModoDeFuncionamientoTemporario,0
	
	bcf	ModoDeFuncionamientoTemporario,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	CambiarModoDeFuncionamiento
	
	movf	ModoDeFuncionamientoTemporario,W
	xorwf	ModoDeFuncionamiento,W
	
	btfsc	STATUS,Z
	goto	ModoFuncionamiento
	
	btfsc	ModoFermentar
	goto	ConfirmarSalirModoFermentar
	
	btfsc	ModoDeFuncionamientoTemporario,1
	goto	ConfirmarModoFermentar
	
	movf	ModoDeFuncionamientoTemporario,W
	movwf	ModoDeFuncionamiento
	
	Bank1
	movlw	0x08
	movwf	EEADR
	bcf	STATUS,RP0
	movf	ModoDeFuncionamiento,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	goto	FinInt

ConfirmarSalirModoFermentar
	call	LCD_Borra
	
	movlw	Detener
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	FermentacionMaduracion
	call	LCD_MensajeAlto
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	ModoFuncionamiento
	
	movf	ModoDeFuncionamientoTemporario,W
	movwf	ModoDeFuncionamiento
	
	Bank1
	movlw	0x08
	movwf	EEADR
	bcf	STATUS,RP0
	movf	ModoDeFuncionamiento,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	goto	FinInt

ConfirmarModoFermentar
	call	LCD_Borra
	
	movlw	Iniciar
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	FermentacionMaduracion
	call	LCD_MensajeAlto
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	ModoFuncionamiento
	
	clrf	ContadorDias
	clrf	Contador4SegundosParaDias
	clrf	Contador800Segundos
	
	clrf	MinutosEncendido
	clrf	HorasEncendido
	clrf	DiasEncendido
	
	movf	ModoDeFuncionamientoTemporario,W
	movwf	ModoDeFuncionamiento
	
	Bank1
	movlw	0x05
	movwf	EEADR
	bcf	STATUS,RP0
	movf	Contador4SegundosParaDias,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x06
	movwf	EEADR
	bcf	STATUS,RP0
	movf	Contador800Segundos,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x07
	movwf	EEADR
	bcf	STATUS,RP0
	movf	ContadorDias,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x08
	movwf	EEADR
	bcf	STATUS,RP0
	movf	ModoDeFuncionamiento,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x20
	movwf	EEADR
	bcf	STATUS,RP0
	movf	MinutosEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x21
	movwf	EEADR
	bcf	STATUS,RP0
	movf	HorasEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x22
	movwf	EEADR
	bcf	STATUS,RP0
	movf	DiasEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	goto	FinInt

SeleccionarSalida
	call	LCD_Borra
	
	movlw	SalidaPara
	call	LCD_MensajeAlto
	
	call	LCD_Linea2
	
	movlw	Enfriar
	btfsc	SalidaParaCalentar
	movlw	Calentar
	
	call	LCD_MensajeAlto
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	AjustarTempTermostato
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	ModoFuncionamiento
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinInt

	movf	Flags1,W
	movwf	FlagTemp

CambiarSalida
	call	LCD_Linea2
	
	movlw	Enfriar
	btfsc	FlagTemp,5
	movlw	Calentar
	
	call	LCD_MensajeAlto
	
	movlw	.1			; La flechita
	call	LCD_EnviaDato
	
	movlw	' '			; Para 'borrar'
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	goto	SeleccionarSalida
	
	movf	FlagTemp,W
	xorlw	b'00100000'
	movwf	FlagTemp
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	CambiarSalida
	
	movf	FlagTemp,W
	xorlw	b'00100000'
	movwf	Flags1
	
	Bank1
	movlw	0x01
	movwf	EEADR
	bcf	STATUS,RP0
	movf	Flags1,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	goto	SeleccionarSalida

AjustarTempTermostato
	call	LCD_Borra
	
	movlw	TempPunto
	call	LCD_Mensaje
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Termostato
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movf	TempModoTermostatoMSB,W
	movwf	TemperaturaAEscribirEntero
	movf	TempModoTermostatoLSB,W
	movwf	TemperaturaAEscribirDecimal
	
	call	EscribirTemperatura
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Grados
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	AjustarTolerancia
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	SeleccionarSalida
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinInt
	
	movf	TempModoTermostatoMSB,W
	movwf	TemporarioMSB
	movf	TempModoTermostatoLSB,W
	movwf	TemporarioLSB
	
CambiarTempTermostato
	call	LCD_Linea2
	
	movf	TemporarioMSB,W
	movwf	TemperaturaAEscribirEntero
	movf	TemporarioLSB,W
	movwf	TemperaturaAEscribirDecimal
	call	EscribirTemperatura
	
	movlw	.1			; La flechita
	call	LCD_EnviaDato
	
	movlw	Grados
	call	LCD_Mensaje
	
	movlw	' '			; Para 'borrar'
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenLosBotonesOPasenUnosMs
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	goto	AjustarTempTermostato
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	call	IncrementarX8Temporario
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	call	DecrementarX8Temporario
	
	movf	TemporarioMSB,0
	xorlw	.35
	btfsc	STATUS,Z
	clrf	TemporarioLSB
	
	movf	TemporarioMSB,0
	xorlw	b'11101011'	; -21
	btfsc	STATUS,Z
	call	IncrementarX8Temporario
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	CambiarTempTermostato
	
	Bank1
	movlw	0x0B
	movwf	EEADR
	bcf	STATUS,RP0
	movf	TemporarioLSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x0C
	movwf	EEADR
	bcf	STATUS,RP0
	movf	TemporarioMSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	movf	TemporarioLSB,W
	movwf	TempModoTermostatoLSB
	movf	TemporarioMSB,W
	movwf	TempModoTermostatoMSB
	goto	AjustarTempTermostato

AjustarTolerancia
	call	LCD_Borra
	
	movlw	Tolerancia
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	.177
	;movlw	b'00010000'		; +/-
	call	LCD_EnviaDato
	
	movf	DeltaTMSB,W
	movwf	TemperaturaAEscribirEntero
	movf	DeltaTLSB,W
	movwf	TemperaturaAEscribirDecimal
	call	EscribirTemperatura2Decimales
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Grados
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	PreguntaConfigurarPrograma
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	AjustarTempTermostato
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinInt
	
	movf	DeltaTLSB,W
	movwf	TemporarioLSB
	movf	DeltaTMSB,W
	movwf	TemporarioMSB

CambiarTolerancia
	call	LCD_Linea2
	
	movlw	.177
	;movlw	b'00010000'		; +/-
	call	LCD_EnviaDato
	
	movf	TemporarioMSB,W
	movwf	TemperaturaAEscribirEntero
	movf	TemporarioLSB,W
	movwf	TemperaturaAEscribirDecimal
	call	EscribirTemperatura2Decimales
	
	movlw	.1			; La flechita
	call	LCD_EnviaDato
	
	movlw	Grados
	call	LCD_Mensaje
	
	movlw	' '
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenLosBotonesOPasenUnosMs
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	goto	AjustarTolerancia
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	call	IncrementarX4Temporario
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	call	DecrementarX4Temporario
	
	movf	TemporarioMSB,0
	xorlw	.5
	btfsc	STATUS,Z
	clrf	TemporarioLSB
	
	movf	TemporarioMSB,0
	btfsc	STATUS,Z
	call	CortarEn025
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	CambiarTolerancia
	
	Bank1
	movlw	0x09
	movwf	EEADR
	bcf	STATUS,RP0
	movf	TemporarioLSB,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x0A
	movwf	EEADR
	bcf	STATUS,RP0
	movf	TemporarioMSB,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	movf	TemporarioLSB,W
	movwf	DeltaTLSB
	movf	TemporarioMSB,W
	movwf	DeltaTMSB
	goto	AjustarTolerancia

PreguntaConfigurarPrograma
	bsf	Backlight
	clrf	ContadorParaApagarBacklight
	
	bsf	PCLATH,3
	
;	btfsc	ControlDeMacerado
;	goto	PreguntaConfigurarProgramaMacerado
	
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Configurar
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	Programa
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	Info0
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	AjustarTolerancia
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinInt

Ajustar1eraFermentacion
	movlw	Primera
	movwf	Texto1
	
	movlw	Fermentacion
	movwf	Texto2
	
	movf	Dias1raF,W
	movwf	Texto3
	
	movf	Temp1raFLSB,W
	movwf	TemporarioLSB
	movwf	TemperaturaAEscribirDecimal
	
	movf	Temp1raFMSB,W
	movwf	TemporarioMSB
	movwf	TemperaturaAEscribirEntero
	
	call	DibujarAjustePrograma
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	Ajustar2daFermentacion
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	CancelarAjuste
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinIntRestaurandoCambios
	
	call	AjustarValores
	
	xorlw	.255
	btfsc	STATUS,Z
	goto	Ajustar1eraFermentacion
	
	movf	Texto3,W
	movwf	Dias1raF
	
	movf	TemporarioLSB,W
	movwf	Temp1raFLSB
	movf	TemporarioMSB,W
	movwf	Temp1raFMSB
	
	goto	Ajustar1eraFermentacion

Ajustar2daFermentacion
	movlw	Segunda
	movwf	Texto1
	
	movlw	Fermentacion
	movwf	Texto2
	
	movf	Dias2daF,W
	movwf	Texto3
		
	movf	Temp2daFLSB,W
	movwf	TemporarioLSB
	movwf	TemperaturaAEscribirDecimal
	
	movf	Temp2daFMSB,W
	movwf	TemporarioMSB
	movwf	TemperaturaAEscribirEntero
	
	call	DibujarAjustePrograma
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	AjustarMaduracion
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	Ajustar1eraFermentacion
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinIntRestaurandoCambios
	
	call	AjustarValores
	
	xorlw	.255
	btfsc	STATUS,Z
	goto	Ajustar2daFermentacion
	
	movf	Texto3,W
	movwf	Dias2daF
	
	movf	TemporarioLSB,W
	movwf	Temp2daFLSB
	movf	TemporarioMSB,W
	movwf	Temp2daFMSB
	
	goto	Ajustar2daFermentacion

AjustarMaduracion
	movlw	Maduracion
	movwf	Texto1
	
	movlw	Nada
	movwf	Texto2
	
	movf	DiasMadur,W
	movwf	Texto3
	
	movf	TempMaduracionLSB,W
	movwf	TemporarioLSB
	movwf	TemperaturaAEscribirDecimal
	
	movf	TempMaduracionMSB,W
	movwf	TemporarioMSB
	movwf	TemperaturaAEscribirEntero
	
	call	DibujarAjustePrograma
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	PreguntaSiPasaAModoTermostato
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	Ajustar2daFermentacion
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinIntRestaurandoCambios
	
	call	AjustarValores
	
	xorlw	.255
	btfsc	STATUS,Z
	goto	AjustarMaduracion
	
	movf	Texto3,W
	movwf	DiasMadur
	
	movf	TemporarioLSB,W
	movwf	TempMaduracionLSB
	movf	TemporarioMSB,W
	movwf	TempMaduracionMSB
	
	goto	AjustarMaduracion

PreguntaSiPasaAModoTermostato
	call	LCD_Borra
	
	movlw	DespuesTermostato
	call	LCD_Mensaje
	
	movlw	.2
	call	LCD_PosicionLinea2
	
	movlw	Si
	btfss	LuegoTermostato
	movlw	No
	call	LCD_Mensaje
	
	movlw	.13
	call	LCD_PosicionLinea2
	
	movf	TempPasarAModoTermostatoLSB,W
	movwf	TemperaturaAEscribirDecimal
	movwf	TemporarioLSB
	
	movf	TempPasarAModoTermostatoMSB,W
	movwf	TemperaturaAEscribirEntero
	movwf	TemporarioMSB
	
	movlw	0x04			; Dere a Izda
	call	LCD_EnviaComando
	
	movlw	GradosAlReves
	call	LCD_Mensaje
	
	movlw	' '
	call	LCD_EnviaDato
	
	call	EscribirTemperaturaDeAtrasParaAdelante
	
	movlw	0x06			; Izda a Dere
	call	LCD_EnviaComando
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	SalirDelAjuste
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	AjustarMaduracion
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinIntRestaurandoCambios
	
	movf	Flags,W
	movwf	Texto1
	
AjustarSiPasaAModoTermostato
	movlw	.2
	call	LCD_PosicionLinea2
	
	movlw	Si
	btfss	Texto1,6
	movlw	No
	call	LCD_Mensaje
	
	movlw	.1
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	goto	PreguntaSiPasaAModoTermostato
	
	movf	Botones,0
	xorlw	b'00000011'
	btfsc	STATUS,Z
	goto	AjustarSiPasaAModoTermostatoTemperatura
	
	movlw	b'01000000'
	xorwf	Texto1,F
	
	goto	AjustarSiPasaAModoTermostato
	
AjustarSiPasaAModoTermostatoTemperatura
	movlw	.4
	call	LCD_PosicionLinea2
	
	movlw	' '
	call	LCD_EnviaDato
	
	movf	TemporarioLSB,W
	movwf	TemperaturaAEscribirDecimal
	movf	TemporarioMSB,W
	movwf	TemperaturaAEscribirEntero
	
	movlw	.13
	call	LCD_PosicionLinea2
	
	movlw	0x04			; Dere a Izda
	call	LCD_EnviaComando
	
	movlw	GradosAlReves
	call	LCD_Mensaje
	
	movlw	.1
	call	LCD_EnviaDato
	
	call	EscribirTemperaturaDeAtrasParaAdelante
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	0x06			; Izda a Dere
	call	LCD_EnviaComando
	
	call	EsperarQueSueltenLosBotonesOPasenUnosMs
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	goto	PreguntaSiPasaAModoTermostato
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	call	IncrementarX8Temporario
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	call	DecrementarX8Temporario
	
	movf	TemporarioMSB,W
	xorlw	.35
	btfsc	STATUS,Z
	clrf	TemporarioLSB
	
	movf	TemporarioMSB,W
	xorlw	b'11110000'
	btfsc	STATUS,Z
	call	IncrementarX8Temporario
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	AjustarSiPasaAModoTermostatoTemperatura
	
	movf	Texto1,W
	movwf	Flags
	
	movf	TemporarioLSB,W
	movwf	TempPasarAModoTermostatoLSB
	movf	TemporarioMSB,W
	movwf	TempPasarAModoTermostatoMSB
	
	goto	PreguntaSiPasaAModoTermostato

SalirDelAjuste
	call	LCD_Borra
	
	movlw	Guardar
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	LosAjustes
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	CancelarAjuste
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	PreguntaSiPasaAModoTermostato
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinIntRestaurandoCambios
	
	bsf	PCLATH,3
	call	MandarConfiguracionAEEPROM
	goto	FinInt

CancelarAjuste
	call	LCD_Borra
	
	movlw	Cancelar
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	LosAjustes
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	Ajustar1eraFermentacion
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	SalirDelAjuste

FinIntRestaurandoCambios
	bsf	PCLATH,3
	call	RestaurarCambios

FinInt
	bsf	PCLATH,3
	
	btfsc	PIR1,TMR1IF
	goto	DesbordoElTimer1
	
	bcf	PCLATH,3
	
	movf	PORTB,W
	bcf	INTCON,RBIF
	
	movf	PCLATH_Temp,W
	movwf	PCLATH
	
;	movf	PCL_Temp,W
;	movwf	PCL
	
	swapf	STATUS_Temp,W
	movwf	STATUS
	
	swapf	W_Temp,F
	swapf	W_Temp,W
	retfie

;-------------------------------------------------------------------------------
;	Inicio programa
;-------------------------------------------------------------------------------
Inicio
	bsf	PCLATH,3
	call	Inicializar

ZZZ
	bsf	INTCON,7	; ¿Afuera de ZZZ?
	
	incf	ContadorParaApagarBacklight
	
	btfsc	ContadorParaApagarBacklight,2
	call	VerSiApagoElBacklight
	
	sleep			; Guarda que la interrupcion lo saca de sleep y tarda 1, 2 instrucciones en saltar a 0x04 ¿se???
	
	btfss	STATUS,NOT_TO	; 0 => Desbordó el wdt
	goto	ZZZ
	
	call	LCD_Borra
	call	LCD_Linea2
	
	btfsc	ProcesoFinalizado
	goto	TerminoElProceso
	
;	btfsc	ControlDeMacerado
;	goto	ControlDeMaceradoZZZ
	
	btfsc	ModoTermostato
	goto	EstaTermostato
	
	btfsc	ModoFermentar
	goto	EstaFermentando
	
	btfsc	ModoApagado
	goto	EstaApagado
	
	bcf	ActivarSalida
	movlw	TError
	call	LCD_Mensaje
	goto	ZZZ

EstaFermentando
	bsf	PCLATH,3
	call	VerEnQueFaseEsta
	
	bsf	PCLATH,3
	call	VerSiParaOArranca
	bcf	PCLATH,3
	
	call	EscribirPrimerRenglonCompleto
	goto	ZZZ

EstaTermostato
	movf	TempModoTermostatoLSB,W
	movwf	TemperaturaBuscadaLSB
	movf	TempModoTermostatoMSB,W
	movwf	TemperaturaBuscadaMSB
	
	bsf	PCLATH,3
	call	VerSiParaOArranca
	bcf	PCLATH,3
	
	movlw	Modo
	call	LCD_Mensaje
	
	movlw	.6
	call	LCD_PosicionLinea2
	
	movlw	Termostato
	call	LCD_Mensaje
	
	call	EscribirPrimerRenglonCompleto
	goto	ZZZ

TerminoElProceso
	bsf	PCLATH,3
	call	Parar
	bcf	PCLATH,3
	
	clrf	ModoDeFuncionamiento
	bsf	ModoApagado
	
	movlw	FinDelProceso
	call	LCD_Mensaje
	
	call	EscribirPrimerRenglonParcial
	goto	ZZZ

EstaApagado
	bsf	PCLATH,3
	call	Parar
	bcf	PCLATH,3
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Apagado
	call	LCD_Mensaje
	
	call	EscribirPrimerRenglonParcial
	goto	ZZZ

;ControlDeMaceradoZZZ
;	btfsc	ModoFermentar
;	goto	EstaMacerando
	
;	btfsc	ModoApagado
;	goto	EstaApagado
	
;	bcf	ActivarSalida
;	movlw	TError
;	call	LCD_Mensaje
;	goto	ZZZ

;EstaMacerando
;	bsf	PCLATH,3
;	call	VerEnQueFaseEstaMacerado
	
;	bsf	PCLATH,3
;	call	VerSiParaOArrancaMacerando
;	bcf	PCLATH,3
	
;	call	EscribirPrimerRenglonCompleto
;	goto	ZZZ

;-------------------------------------------------------------------------------
;	Subrutinas 1-Wire
;-------------------------------------------------------------------------------
Bus1Linea_AltaImpedancia
	Bank1
	bsf	Bus1Linea		; La lםnea como entrada.
	bcf	STATUS,RP0
	return

Bus1Linea_Bajo
	Bank1
	bcf	Bus1Linea		; La lםnea como salida.
	bcf	STATUS,RP0
	bcf	Bus1Linea		; Pasa a bajo.
	return

Bus1Linea_Reset
	call	Bus1Linea_AltaImpedancia	; Empieza en alto para conseguir el flanco de bajada con la siguiente instrucciףn.
	call	Bus1Linea_Bajo			; El bus de 1 lea en bajo durante 500 µs.
	call	Retardo_500micros
	call	Bus1Linea_AltaImpedancia	; Permanece en alta impedancia (nivel alto por
	call	Retardo_20micros		; las resistencias de Pull-Up) durante unos .
Bus1Linea_EsperaLeerBajo
	btfsc	Bus1Linea			; Espera a que el dispositivo le envםe el pulso
	goto	Bus1Linea_EsperaLeerBajo	; en bajo de "Presence".
Bus1Linea_EsperaLeerAlto
	btfss	Bus1Linea			; Y ahora el dispositivo lo debe poner en alto.
	goto	Bus1Linea_EsperaLeerAlto
	return

Bus1Linea_EscribeByte
	movwf	OneWireData			; Guarda el byte a transmitir.
	movlw	.8				; 8 bits a transmitir.
	movwf	OneWireBitCount
	call	Bus1Linea_AltaImpedancia	; Empieza en alta impedancia.
Bus1Linea_EnviaBit
	call	Bus1Linea_Bajo			; Flanco de bajada.
	rrf	OneWireData,F		; Lleva el bit de menor peso LSB al Carry. 
	btfss	STATUS,C			; �Es un "1" el bit a transmitir?.
	goto	Bus1Linea_EnviaCero		; No, pues env un "0".	
Bus1Linea_EnviaUno
	call	Bus1Linea_AltaImpedancia	; Transmite un "1".
	goto	Bus1Linea_Espera50us
Bus1Linea_EnviaCero
	call	Bus1Linea_Bajo			; Transmite un "0".
Bus1Linea_Espera50us
	call	Retardo_50micros
	call	Bus1Linea_AltaImpedancia	; Flanco de subida.
	decfsz	OneWireBitCount,1	; Comprueba que es el timo bit.
	goto	Bus1Linea_EnviaBit		; Como no es el timo bit repite la operaci.
	return

Bus1Linea_LeeByte
	movlw	.8				; 8 bits a recibir.
	movwf	OneWireBitCount
	call	Bus1Linea_AltaImpedancia	; Empieza en alta impedancia.
Bus1Linea_LeeBit
	call	Bus1Linea_Bajo			; Flanco de bajada.
	call	Bus1Linea_AltaImpedancia
	call	Retardo_10micros		; Adiendo las instrucciones anteriores, supone
						; un total de 15 desde el flanco de bajada.
	bcf	STATUS,C			; Ahora lee el pin. En principio supone que es 0.
	btfsc	Bus1Linea			; ealmente es cero?.
	bsf	STATUS,C			; No, pues cambia a uno.
	rrf	OneWireData,1		; Introduce el bit en el registro.
	call	Retardo_50micros		; Espera.
	call	Bus1Linea_AltaImpedancia	; Flanco de subida del pulso positivo.
	decfsz 	OneWireBitCount,1	; Comprueba que es el timo bit.
	goto	Bus1Linea_LeeBit		; Si no es el timo bit pasa a leer el siguiente.
	movf	OneWireData,W		; El resultado en el registro W.
	return

;-------------------------------------------------------------------------------
;	Subrutinas DS1820
;-------------------------------------------------------------------------------
DS1820ConvertirTemperatura
	call	Bus1Linea_Reset
	movlw	DS1820_Skip_ROM_Command
	call	Bus1Linea_EscribeByte
	movlw	DS1820_Convert_T_Command
	call	Bus1Linea_EscribeByte
	return

; DS1820Configurar
	; call	Bus1Linea_Reset
	; movlw	DS1820_Skip_ROM_Command
	; call	Bus1Linea_EscribeByte
	; call	DS1820_Write_Scratchpad_Command
	; call	Bus1Linea_EscribeByte
	; movlw	.12
	; call	Bus1Linea_EscribeByte
	; movlw	.10
	; call	Bus1Linea_EscribeByte
	; movlw	b'00011111'
	; call	Bus1Linea_EscribeByte
	; return

DS1820LeerTemperatura
	call	Bus1Linea_Reset			; Primero los pulsos de "Reset" y "Presence".
	movlw	DS1820_Skip_ROM_Command 	; Salta los comandos de ROM ya que hay
	call	Bus1Linea_EscribeByte		; conectado un ico dispositivo al bus.
	movlw	DS1820_Read_Scratchpad_Command	; Va a leer la primera posici de memoria,
	call	Bus1Linea_EscribeByte		; donde se localiza el valor de la temperatura.
	call	Bus1Linea_LeeByte		; Lee la temperatura en 
	movwf	TemperaturaLeidaLSB		; complemento a 2.
	call	Bus1Linea_LeeByte		; Ahora lee el segundo byte que indica el signo
	movwf	TemperaturaLeidaMSB		; de la temperatura.
	call	Bus1Linea_LeeByte
	call	Bus1Linea_LeeByte
	call	Bus1Linea_LeeByte
	call	Bus1Linea_LeeByte
	call	Bus1Linea_LeeByte
	call	Bus1Linea_LeeByte
	call	Bus1Linea_LeeByte
	movwf	CRC
	call	Bus1Linea_Reset			; Y para terminar, resetea el dispositivo.
	return

;-------------------------------------------------------------------------------
;	Subrutinas botones
;-------------------------------------------------------------------------------
EsperarQueSueltenLosBotonesOPasenUnosMs
	movlw	D'75'
	movwf	CounterB
	movlw	D'100'
	movwf	CounterA

BucleDeUnosMs
	clrw
	
	btfss	BotonSubir
	addlw	b'00000001'
	
	btfss	BotonBajar
	addlw	b'00000010'
	
	btfsc	STATUS,Z
	goto	EsperarQueSueltenBotones	;Este goto es nada más por el retardo de EsperarQueSueltenBotones
	
	decfsz	CounterA,F
	goto	BucleDeUnosMs
	decfsz	CounterB,F
	goto	BucleDeUnosMs
	return

EsperarQueSueltenBotones
	clrwdt
	
	call	Retardo_30ms
	
	btfss	BotonSubir
	goto	EsperarQueSueltenBotones
	btfss	BotonBajar
	goto	EsperarQueSueltenBotones
	return

RevisarBotones
	clrf	Botones
	
	call	Retardo_30ms
	
	btfss	BotonSubir
	bsf	Botones,0
	
	btfss	BotonBajar
	bsf	Botones,1
	return

EsperarBotonesVariosSegundos
	call	Retardo_30ms
	
	clrf	Botones
	
	movlw	D'100'
	movwf	CounterC
	movlw	D'150'
	movwf	CounterB
	movlw	D'150'
	movwf	CounterA

BucleEsperarBotonesVariosSegundos
	clrwdt
	btfss	BotonSubir
	goto	HayAlgoApretado
	btfss	BotonBajar
	goto	HayAlgoApretado
	
	decfsz	CounterA,F
	goto	BucleEsperarBotonesVariosSegundos
	
	decfsz	CounterB,F
	goto	BucleEsperarBotonesVariosSegundos
	
	decfsz	CounterC,F
	goto	BucleEsperarBotonesVariosSegundos
	return

EsperarBotonesUnosSegundos
	clrf	Botones
	
	call	Retardo_30ms
	
	movlw	D'10'
	movwf	CounterC
	movlw	D'15'
	movwf	CounterB
	movlw	D'20'
	movwf	CounterA

BucleEsperarBotonesUnosSegundos
	clrwdt
	btfss	BotonSubir
	goto	HayAlgoApretado
	btfss	BotonBajar
	goto	HayAlgoApretado
	
	decfsz	CounterA,F
	goto	BucleEsperarBotonesUnosSegundos
	
	decfsz	CounterB,F
	goto	BucleEsperarBotonesUnosSegundos
	
	decfsz	CounterC,F
	goto	BucleEsperarBotonesUnosSegundos
	return

HayAlgoApretado
	btfss	BotonSubir
	bsf	Botones,0
	
	btfss	BotonBajar
	bsf	Botones,1
	
	bsf	PCLATH,3
	call	Beep
	
	return

SeAccionoUnBoton
	bsf	PCLATH,3
	call	Beep
	
	btfss	ProcesoFinalizado
	return
	
	bcf	ProcesoFinalizado
	
	Bank1
	movlw	0x00
	movwf	EEADR
	bcf	STATUS,RP0
	movf	Flags,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	clrf	ModoDeFuncionamiento
	bsf	ModoApagado
	
	Bank1
	movlw	0x08
	movwf	EEADR
	bcf	STATUS,RP0
	movf	ModoDeFuncionamiento,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	return

;-------------------------------------------------------------------------------
;	Subrutinas varias
;-------------------------------------------------------------------------------
LCD_MensajeAlto
	bsf	PCLATH,0
	call	LCD_Mensaje
	bcf	PCLATH,0
	return

CortarEn025
	movf	TemporarioLSB,0
	btfss	STATUS,Z
	return
	
	movlw	b'00000100'
	movwf	TemporarioLSB
	return

IncrementarX4Temporario
	movlw	.4
	addwf	TemporarioLSB
	
	btfss	STATUS,DC
	return
	
	incf	TemporarioMSB,F
	
	movlw	b'00001111'
	andwf	TemporarioLSB,F
	return

DecrementarX4Temporario
	movlw	.4
	subwf	TemporarioLSB
	
	btfsc	STATUS,DC
	return
	
	decf	TemporarioMSB,F
	
	movlw	b'00001111'
	andwf	TemporarioLSB,F
	return

IncrementarX8Temporario
	movlw	.8
	addwf	TemporarioLSB,F
	
	btfss	STATUS,DC
	return
	
	incf	TemporarioMSB,F
	
	movlw	b'00001111'
	andwf	TemporarioLSB,F
	return

DecrementarX8Temporario
	movlw	.8
	subwf	TemporarioLSB,F
	
	btfsc	STATUS,DC
	return
	
	decf	TemporarioMSB,F
	
	movlw	b'00001111'
	andwf	TemporarioLSB,F
	return

VerSiApagoElBacklight
	btfss	SeDeseaElBacklightPrendido
	bcf	Backlight
	return

CambiarBacklight
	bsf	SeDeseaElBacklightPrendido
	
	btfsc	Backlight
	bcf	SeDeseaElBacklightPrendido
	
	btfss	SeDeseaElBacklightPrendido
	bcf	Backlight
	
	btfsc	SeDeseaElBacklightPrendido
	bsf	Backlight
	
	call	EsperarQueSueltenBotones
	return

EscribirPrimerRenglonCompleto
	call	LCD_Linea1
	
	movlw	0x06			; Izda a Dere
	call	LCD_EnviaComando
	
	movlw	' '
	call	LCD_EnviaDato
	
	call	EscribirTemperaturaActual
	
	movlw	Grados
	call	LCD_Mensaje
	
	movlw	.14
	call	LCD_PosicionLinea1
	
	movlw	0x04			; Dere a Izda
	call	LCD_EnviaComando
	
	movf	TemperaturaBuscadaMSB,W
	movwf	TemperaturaAEscribirEntero
	
	movf	TemperaturaBuscadaLSB,W
	movwf	TemperaturaAEscribirDecimal
	
	movlw	GradosAlReves
	call	LCD_Mensaje
	
	call	EscribirTemperaturaDeAtrasParaAdelante
	return

EscribirPrimerRenglonParcial
	movlw	.1
	call	LCD_PosicionLinea1
	
	call	EscribirTemperaturaActual
	
	movlw	Grados
	call	LCD_Mensaje
	return

AjustarValores
	call	LCD_Linea2
	
	movf	Texto3,W
	call	EscribirEntero
	
	movlw	.1
	call	LCD_EnviaDato
	
	movlw	Dias
	call	LCD_Mensaje
	
	call	EsperarQueSueltenLosBotonesOPasenUnosMs
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	retlw	.255
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	incf	Texto3,F
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	decf	Texto3,F
	
	movf	Texto3,0
	xorlw	.86
	btfsc	STATUS,Z
	decf	Texto3,F
	
	movf	Texto3,0
	xorlw	.255
	btfsc	STATUS,Z
	incf	Texto3,F
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	AjustarValores
	
	call	LCD_Linea2
	
	movf	Texto3,W
	call	EscribirEntero
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Dias
	call	LCD_Mensaje
	
	movlw	0x04			; Dere a Izda
	call	LCD_EnviaComando
	
AjustarSegundoValor
	movlw	.15
	call	LCD_PosicionLinea2
	
	movlw	GradosAlReves
	call	LCD_Mensaje
	
	movlw	.1
	call	LCD_EnviaDato
	
	movf	TemporarioLSB,W
	movwf	TemperaturaAEscribirDecimal
	movf	TemporarioMSB,W
	movwf	TemperaturaAEscribirEntero
	
	call	EscribirTemperaturaDeAtrasParaAdelante
	
	movlw	' '
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenLosBotonesOPasenUnosMs
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	retlw	.255
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	call	IncrementarX8Temporario
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	call	DecrementarX8Temporario
	
	movf	TemporarioMSB,0
	xorlw	.35
	btfsc	STATUS,Z
	clrf	TemporarioLSB
	
	movf	TemporarioMSB,0
	xorlw	b'11110000'
	btfsc	STATUS,Z
	call	IncrementarX8Temporario
	
	movf	Botones,0
	xorlw	b'00000011'
	btfsc	STATUS,Z
	retlw	.0
	goto	AjustarSegundoValor

DibujarAjustePrograma
	call	LCD_Borra
	
	movf	Texto1,W
	call	LCD_Mensaje
	
	movlw	' '
	call	LCD_EnviaDato
	
	movf	Texto2,W
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movf	Texto3,W
	call	EscribirEntero
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Dias
	call	LCD_Mensaje
	
	movlw	.15
	call	LCD_PosicionLinea2
	
	movlw	0x04			; Dere a Izda
	call	LCD_EnviaComando
	
	movlw	GradosAlReves
	call	LCD_Mensaje
	
	movlw	' '
	call	LCD_EnviaDato
	
	call	EscribirTemperaturaDeAtrasParaAdelante
	
	movlw	0x06			; Izda a Dere
	call	LCD_EnviaComando
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	return

DibujarMenuInfo
	call	LCD_Borra
	
	movlw	Restante
	call	LCD_Mensaje
	
	movlw	' '
	call	LCD_EnviaDato
	
	movf	Texto1,W
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movf	Texto2,W
	call	EscribirEntero
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Dias
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	return

EscribirEntero
	call	ConvertirABCD
	
	movf	Centenas,W
	btfss	STATUS,Z
	goto	EscribirEnteroHayCentenas
	
	movf	Decenas,W
	btfsc	STATUS,Z
	movlw	b'11110000'	; ' ' - '0'
	
EscribirEnteroVuelveDeHayCentenas
	call	LCD_EnviaNumero
	
	movf	Unidades,0
	call	LCD_EnviaNumero
	return

EscribirEnteroHayCentenas
	call	LCD_EnviaNumero
	movf	Decenas,W
	goto	EscribirEnteroVuelveDeHayCentenas
	
EscribirTemperaturaActual
	movf	TemperaturaEntero,W
	movwf	TemperaturaAEscribirEntero
	movf	TemperaturaDecimal,W
	movwf	TemperaturaAEscribirDecimal
	
EscribirTemperatura
	bcf	LaTempEsNegativa
	
	btfsc	TemperaturaAEscribirEntero,7
	call	LaTempAEscribirEsNegativa
	
	movlw	'-'
	btfsc	LaTempEsNegativa
	call	LCD_EnviaDato
	
	movf	TemperaturaAEscribirEntero,W
	call	ConvertirABCD
	
	movf	Centenas,W
	btfss	STATUS,Z
	goto	TenemosCentenas
	
	movf	Decenas,W
	btfss	STATUS,Z
	call	LCD_EnviaNumero
	
VuelvoDeTenemosCentenas
	movf	Unidades,W
	call	LCD_EnviaNumero
	
	movlw	'.'
	call	LCD_EnviaDato
	
	movf	TemperaturaAEscribirDecimal,W
	
	bsf	PCLATH,0		;La tabla está fuera de 0xFF
	call	TablaCalcularElDecimal
	bcf	PCLATH,0
	
	call	LCD_EnviaNumero
	
	return
	
TenemosCentenas
	call	LCD_EnviaNumero
	
	movf	Decenas,W
	call	LCD_EnviaNumero
	goto	VuelvoDeTenemosCentenas

EscribirTemperaturaDeAtrasParaAdelante
	bcf	LaTempEsNegativa
	
	btfsc	TemperaturaAEscribirEntero,7
	call	LaTempAEscribirEsNegativa
	
	bsf	PCLATH,0		;La tabla está fuera de 0xFF
	movf	TemperaturaAEscribirDecimal,W
	call	TablaCalcularElDecimal
	bcf	PCLATH,0
	call	LCD_EnviaNumero
	
	movlw	'.'
	call	LCD_EnviaDato
	
	movf	TemperaturaAEscribirEntero,W
	call	ConvertirABCD
	
	movf	Unidades,W
	call	LCD_EnviaNumero
	
	movf	Centenas,W
	btfss	STATUS,Z
	goto	TenemosCentenasParaAtras
	
	movf	Decenas,W
	btfss	STATUS,Z
	call	LCD_EnviaNumero
	
	movlw	'-'
	btfsc	LaTempEsNegativa
	call	LCD_EnviaDato
	return
	
TenemosCentenasParaAtras
	movf	Decenas,W
	call	LCD_EnviaNumero
	movf	Centenas,W
	call	LCD_EnviaNumero
	
	movlw	'-'
	btfsc	LaTempEsNegativa
	call	LCD_EnviaDato
	return

EscribirTemperatura2Decimales
	movf	TemperaturaAEscribirEntero,W
	call	ConvertirABCD
	
	movf	Decenas,W
	btfss	STATUS,Z
	call	LCD_EnviaNumero
	
	movf	Unidades,W
	call	LCD_EnviaNumero
	
	movlw	'.'
	call	LCD_EnviaDato
	
	movf	TemperaturaAEscribirDecimal,W
	
	bsf	PCLATH,0			;La tabla está fuera de 0xFF
	call	TablaCalcularPrimerDecimal
	bcf	PCLATH,0
	
	call	LCD_EnviaNumero
	
	movf	TemperaturaAEscribirDecimal,W
	
	bsf	PCLATH,0			;La tabla está fuera de 0xFF
	call	TablaCalcularSegundoDecimal
	bcf	PCLATH,0
	
	call	LCD_EnviaNumero
	return

LaTempAEscribirEsNegativa
	bsf	LaTempEsNegativa
	
	comf	TemperaturaAEscribirEntero,F
	
	comf	TemperaturaAEscribirDecimal,F
	movlw	.1
	addwf	TemperaturaAEscribirDecimal,F	; incf no afecta DC
	
	btfsc	STATUS,DC
	incf	TemperaturaAEscribirEntero,F
	
	movlw	b'00001111'
	andwf	TemperaturaAEscribirDecimal,F
	return

;-------------------------------------------------------------------------------
;	Subrutinas HD47780
;-------------------------------------------------------------------------------
ConvertirABCD
	clrf	Centenas
	clrf	Decenas
	movwf	Unidades
BCD_Resta10
	movlw	.10
	subwf	Unidades,W
	btfss	STATUS,C
	return
BCD_IncrementaDecenas
	movwf	Unidades
	incf	Decenas,F
	movlw	.10
	subwf	Decenas,W
	btfss	STATUS,C
	goto	BCD_Resta10
BCD_IncrementaCentenas
	clrf	Decenas
	incf	Centenas,F
	goto	BCD_Resta10

LCD_EscribeLCD
	andlw	b'11110000'		; Se queda con el nibble alto del dato que es el
	movwf	LCDData		; que hay que enviar y lo guarda.
	
	btfsc	LCDData,4
	bsf	LCDData,2
	
	bcf	LCDData,4
	
	btfsc	LCDData,5
	bsf	LCDData,3
	
	bcf	LCDData,5
	
	movf	LCDDataPort,W		; Lee la informaci actual de la parte baja
	andlw	b'00110011'		; del Puerto B, que no se debe alterar.
	
	iorwf	LCDData,F		; Enviarla parte alta del dato de entrada y en la parte baja lo que hab antes en el puerto.
	
	movf	LCDData,W		; Recupera el dato a enviar.
	movwf	LCDDataPort		; Env el dato al mulo LCD.
	
	bsf	LCDEPin		; Permite funcionamiento del LCD mediante un peque pulso y termina impidiendo el funcionamiento del LCD.
	call	Retardo_50micros
	bcf	LCDEPin
	return

LCD_CursorIncr				; Cursor en modo incrementar.
	movlw	b'00000110'
	goto	LCD_EnviaComando
LCD_Linea1				; Cursor al principio de la Lea 1.
	movlw	b'10000000'		; Direcci 00h de la DDRAM
	goto	LCD_EnviaComando
LCD_Linea2				; Cursor al principio de la Lםnea 2.
	movlw	b'11000000'		; Direc 40h de la DDRAM
	goto	LCD_EnviaComando
LCD_PosicionLinea1			; Cursor a posici de la Lea 1, a partir de la
	iorlw	b'10000000'		; direcci 00h de la DDRAM m el valor del
	goto	LCD_EnviaComando	; registro W.
LCD_PosicionLinea2			; Cursor a posici de la Lea 2, a partir de la
	iorlw	b'11000000'		; direcci 40h de la DDRAM m el valor del
	goto	LCD_EnviaComando	; registro W.
LCD_CursorOFF				; Pantalla encendida y cursor apagado.
	movlw	b'00001100'
	goto	LCD_EnviaComando
LCD_Borra				; Borra toda la pantalla, memoria DDRAM y pone el
	movlw	b'00000001'		; cursor a principio de la lea 1.
	goto	LCD_EnviaComando
LCD_2Lineas4Bits5x7			; Define la pantalla de 2 leas, con caracteres
	movlw	b'00101000'		; de 5x7 puntos y conexi al PIC mediante bus de
	;	goto	LCD_EnviaComando	; 4 bits. 

LCD_EnviaComando
	bcf	LCDRSPin		; Activa el Modo Comando, poniendo RS=0.
	goto	LCD_Envia

LCD_EnviaNumero
	addlw	'0'

LCD_EnviaDato
	bsf	LCDRSPin		; Activa el "Modo Dato", poniendo RS=1.

LCD_Envia
	movwf	LCDTemporario		; Guarda el dato a enviar.
	call	LCD_EscribeLCD		; Primero env el nibble alto.
	swapf	LCDTemporario,W		; Ahora env el nibble bajo. Para ello pasa el nibble bajo del dato a enviar a parte alta del byte.
	call	LCD_EscribeLCD		; Se env al visualizador LCD.
	btfss	LCDRSPin		; Debe garantizar una correcta escritura manteniendo
	call	Retardo_2ms		; 2 ms en modo comando y 50  en modo cacter.
	call	Retardo_50micros
	return

LCD_Mensaje
	movwf	LCD_ApuntaCaracter	; Posici del primer carter del mensaje.
	movlw	Mensajes		; Halla la posici relativa del primer carter
	subwf	LCD_ApuntaCaracter,F	; del mensaje respecto de etiqueta "Mensajes".
	decf	LCD_ApuntaCaracter,F	; Compensa la posici que ocupa "addwf PCL,F".
LCD_VisualizaOtroCaracter
	movf	LCD_ApuntaCaracter,W
	call	Mensajes		; Obtiene el cigo ASCII del carter apuntado.
	movwf	LCD_ValorCaracter	; Guarda el valor de carter.
	movf	LCD_ValorCaracter,F	; Lo ico que hace es posicionar flag Z. En caso
	btfsc	STATUS,Z		; que sea "0x00", que es cףdigo indicador final	
	return				; de mensaje, sale fuera.
LCD_NoUltimoCaracter
	call	LCD_EnviaDato		; Visualiza el carter ASCII leo.
	incf	LCD_ApuntaCaracter,F	; Apunta a la posici del siguiente carter.
	goto	LCD_VisualizaOtroCaracter

;-------------------------------------------------------------------------------
;	Subrutinas EEPROM
;-------------------------------------------------------------------------------
EEPROM_LeeDato
	bsf	STATUS,RP0
	bcf	STATUS,RP1
	movwf	EEADR
	bsf	EECON1,RD
	btfsc	EECON1,RD
	goto	$-1
	movf	EEDATA,W
	bcf	STATUS,RP0
	return

EEPROM_EscribeDato
	bcf	STATUS,RP1
	bsf	STATUS,RP0
	movwf	EEDATA
	bsf	EECON1,WREN
	movlw	0x55
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	btfsc	EECON1,WR
	goto	$-1
	bcf	EECON1,WREN
	bcf	EECON1,EEIF
	bcf	STATUS,RP0
	bcf	STATUS,RP1
	return

;-------------------------------------------------------------------------------
;	Subrutinas retardos
;-------------------------------------------------------------------------------
Retardo_30ms
		movlw	D'39'
		movwf	CounterB
		movlw	D'244'
		movwf	CounterA
loop10
        decfsz	CounterA,1
		goto	loop10
		decfsz	CounterB,1
		goto	loop10
		retlw	0

Retardo_20ms
		movlw	D'26'
		movwf	CounterB
		movlw	D'247'
		movwf	CounterA
loop0
        decfsz	CounterA,1
		goto	loop0
		decfsz	CounterB,1
		goto	loop0
		retlw	0

Retardo_2ms
		movlw	D'3'
		movwf	CounterB
		movlw	D'151'
		movwf	CounterA
loop2
        decfsz	CounterA,1
		goto	loop2
		decfsz	CounterB,1
		goto	loop2
		retlw	0

;Retardo_200micros			; =0.2ms
;		movlw	D'65'
;		movwf	CounterA
;loop3
;       decfsz	CounterA,1
;		goto	loop3
;		retlw	0

Retardo_50micros			; =0.05ms
		movlw	D'15'
		movwf	CounterA
loop4
        decfsz	CounterA,1
		goto	loop4
		retlw	0

Retardo_20micros
		movlw	D'5'
		movwf	CounterA
loop6
        decfsz	CounterA,1
		goto	loop6
		retlw	0

Retardo_10micros
		movlw	D'1'
		movwf	CounterA
loop7
        decfsz	CounterA,1
		goto	loop7
		retlw	0

Retardo_500micros
		movlw	D'165'
		movwf	CounterA
loop8
        decfsz	CounterA,1
		goto	loop8
		retlw	0

;-------------------------------------------------------------------------------
;	Memoria alta
;-------------------------------------------------------------------------------
org 0x800
Inicializar
	bcf	PCLATH,3
	
	movlw	0x00
	call	EEPROM_LeeDato
	movwf	Flags
	call	Retardo_20ms
	
	movlw	0x01
	call	EEPROM_LeeDato
	movwf	Flags1
	call	Retardo_20ms
	
	Bank0
	
	clrf	PORTA
	clrf	PORTB
	
	movlw	b'00000111' 		; Desactivo los comparadores
	movwf	CMCON
	
	movlw	b'00011111'
	movwf	T1CON
	
	clrf	TMR1L
	clrf	TMR1H
	
	Bank1
	
	movlw	b'00000001'
	movwf	PIE1
	
	movlw	b'00001111'
	movwf	OPTION_REG		; Configuramos pull-ups, prescaler y demás
	
	movlw	b'00110000'
	movwf	TRISA			; Configuramos el puerto A
	
	movlw	b'11110000'
	movwf	TRISB			; Configuramos el puerto B
	
	Bank0
	
	bsf	Buzzer
	
	bsf	PCLATH,3
	call	Retardo_1s
	bcf	PCLATH,3
	
	bcf	Buzzer
	
	bcf	ActivarSalida
	
;	Cargar configuracion desde EEPROM
	
	movlw	0x02
	call	EEPROM_LeeDato
	movwf	Dias1raF
	call	Retardo_20ms
	
	movlw	0x03
	call	EEPROM_LeeDato
	movwf	Dias2daF
	call	Retardo_20ms
	
	movlw	0x04
	call	EEPROM_LeeDato
	movwf	DiasMadur
	call	Retardo_20ms
	
;	movlw	0x05
;	call	EEPROM_LeeDato
;	movwf	Contador4SegundosParaDias
;	call	Retardo_20ms
	
	movlw	0x06
	call	EEPROM_LeeDato
	movwf	Contador800Segundos
	call	Retardo_20ms
	
	movlw	0x07
	call	EEPROM_LeeDato
	movwf	ContadorDias
	call	Retardo_20ms
	
	movlw	0x08
	call	EEPROM_LeeDato
	movwf	ModoDeFuncionamiento
	call	Retardo_20ms
	
	movlw	0x09
	call	EEPROM_LeeDato
	movwf	DeltaTLSB
	call	Retardo_20ms
	
	movlw	0x0A
	call	EEPROM_LeeDato
	movwf	DeltaTMSB
	call	Retardo_20ms
	
	movlw	0x0B
	call	EEPROM_LeeDato
	movwf	TempModoTermostatoLSB
	call	Retardo_20ms
	
	movlw	0x0C
	call	EEPROM_LeeDato
	movwf	TempModoTermostatoMSB
	call	Retardo_20ms
	
	movlw	0x0D
	call	EEPROM_LeeDato
	movwf	Temp1raFLSB
	call	Retardo_20ms
	
	movlw	0x0E
	call	EEPROM_LeeDato
	movwf	Temp1raFMSB
	call	Retardo_20ms
	
	movlw	0x0F
	call	EEPROM_LeeDato
	movwf	Temp2daFLSB
	call	Retardo_20ms
	
	movlw	0x10
	call	EEPROM_LeeDato
	movwf	Temp2daFMSB
	call	Retardo_20ms
	
	movlw	0x11
	call	EEPROM_LeeDato
	movwf	TempMaduracionLSB
	call	Retardo_20ms
	
	movlw	0x12
	call	EEPROM_LeeDato
	movwf	TempMaduracionMSB
	call	Retardo_20ms
	
	movlw	0x13
	call	EEPROM_LeeDato
	movwf	TempPasarAModoTermostatoLSB
	call	Retardo_20ms
	
	movlw	0x14
	call	EEPROM_LeeDato
	movwf	TempPasarAModoTermostatoMSB
	call	Retardo_20ms
	
	movlw	0x15
	call	EEPROM_LeeDato
	movwf	CantidadDeEscalones
	call	Retardo_20ms
	
	movlw	0x16
	call	EEPROM_LeeDato
	movwf	MinutosE4
	call	Retardo_20ms
	
	movlw	0x17
	call	EEPROM_LeeDato
	movwf	TempE4LSB
	call	Retardo_20ms
	
	movlw	0x18
	call	EEPROM_LeeDato
	movwf	TempE4MSB
	call	Retardo_20ms
	
	movlw	0x19
	call	EEPROM_LeeDato
	movwf	ContadorMinutos
	call	Retardo_20ms
	
	movlw	0x20
	call	EEPROM_LeeDato
	movwf	MinutosEncendido
	call	Retardo_20ms
	
	movlw	0x21
	call	EEPROM_LeeDato
	movwf	HorasEncendido
	call	Retardo_20ms
	
	movlw	0x22
	call	EEPROM_LeeDato
	movwf	DiasEncendido
	call	Retardo_20ms
	
	bsf	Backlight
	
;	Inicializa LCD
	
	Bank0
	bcf	LCDEPin		; Impide funcionamiento del LCD poniendo E=0.
	bcf 	LCDRSPin		; Activa el Modo Comando poniendo RS=0.
	call	Retardo_20ms
	movlw	b'00110000'
	call	LCD_EscribeLCD		; Escribe el dato en el LCD.
	call	Retardo_20ms
	movlw	b'00110000'
	call	LCD_EscribeLCD
	
	call	Retardo_500micros	;call	Retardo_200micros
	
	movlw	b'00110000'
	call	LCD_EscribeLCD
	movlw	b'00100000'		; Interface de 4 bits.
	call	LCD_EscribeLCD
	
	call	LCD_2Lineas4Bits5x7
	call	LCD_Borra
	call	LCD_CursorOFF
	call	LCD_CursorIncr
	
;	Le mete los caracteres especiales
	
	movlw	b'01001000'
	call	LCD_EnviaComando
	
	movlw	.130			;Flecha - 1
	call	LCD_EnviaDato
	movlw	.134
	call	LCD_EnviaDato
	movlw	.143
	call	LCD_EnviaDato
	movlw	.159
	call	LCD_EnviaDato
	movlw	.143
	call	LCD_EnviaDato
	movlw	.134
	call	LCD_EnviaDato
	movlw	.130
	call	LCD_EnviaDato
	call	LCD_EnviaDato
	
	movlw	.134			;º - 2
	call	LCD_EnviaDato
	movlw	.137
	call	LCD_EnviaDato
	movlw	.137
	call	LCD_EnviaDato
	movlw	.134
	call	LCD_EnviaDato
	movlw	.128
	call	LCD_EnviaDato
	movlw	.128
	call	LCD_EnviaDato
	movlw	.128
	call	LCD_EnviaDato
	call	LCD_EnviaDato
	
	call	LCD_Linea1
	
	bsf	PCLATH,3
	
	btfss	BotonSubir
	call	ResetGeneral
	
	bcf	PCLATH,3
	
	movlw	'I'
	call	LCD_EnviaDato
	movlw	'n'
	call	LCD_EnviaDato
	movlw	'i'
	call	LCD_EnviaDato
	movlw	'c'
	call	LCD_EnviaDato
	movlw	'i'
	call	LCD_EnviaDato
	movlw	'a'
	call	LCD_EnviaDato
	movlw	'n'
	call	LCD_EnviaDato
	movlw	'd'
	call	LCD_EnviaDato
	movlw	'o'
	call	LCD_EnviaDato
	movlw	'.'
	call	LCD_EnviaDato
	movlw	'.'
	call	LCD_EnviaDato
	movlw	'.'
	call	LCD_EnviaDato
	
	call	LCD_Linea2
	
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	
	;	call	DS1820Configurar
	
	bsf	PCLATH,3
	call	Retardo_1s
	bcf	PCLATH,3
	
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	
	call	DS1820ConvertirTemperatura
	
	bsf	PCLATH,3
	call	Retardo_1s
	bcf	PCLATH,3
	
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	
	call	DS1820LeerTemperatura
	
	bsf	PCLATH,3
	call	Retardo_1s
	bcf	PCLATH,3
	
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	
	bsf	PCLATH,3
	call	ProcesarTemperatura
	call	Retardo_1s
	bcf	PCLATH,3
	
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	
	bsf	PCLATH,3
	call	Retardo_1s
	bcf	PCLATH,3
	
	movlw	.255
	call	LCD_EnviaDato
	movlw	.255
	call	LCD_EnviaDato
	
	movlw	b'01001000'
	movwf	INTCON
	
	movlw	.224
	movwf	TMR1H
	
	movlw	.110
	movwf	TemperaturaBuscadaMSB
	clrf	TemperaturaBuscadaLSB
	
	clrf	Contador4Segundos
	clrf	CuatroSegundosEncendido
	
	bsf	PCLATH,3
	
	movlw	.10
	call	Escribir7S
	movlw	.10
	call	Escribir7S
	movlw	.10
	call	Escribir7S
	
	bcf	PCLATH,3
	return

;-------------------------------------------------------------------------------
;	Subrutinas Configurar Macerado
;-------------------------------------------------------------------------------
Info0Macerado
	btfss	ModoFermentar			;Si no está macerando no lo muestra
	goto	ModoFuncionamientoMacerado
	
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Transcurrido
	call	LCD_MensajeAlto
	
	call	LCD_Linea2
	
	movf	ContadorMinutos,W
	call	EscribirEntero
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Mins
	call	LCD_MensajeAlto
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	Info1Macerado
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	PreguntaConfigurarProgramaMacerado
	
	bcf	PCLATH,3
	goto	FinInt

Info1Macerado
	btfss	ModoFermentar			;Si no está macerando no lo muestra
	goto	PreguntaConfigurarProgramaMacerado
	
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Restante
	call	LCD_Mensaje
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Total
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	clrf	SumaTotalMinutos
	
	movf	Dias1raF,W
	addwf	SumaTotalMinutos,F
	
	bsf	PCLATH,3
	
	movf	CantidadDeEscalones,W
	xorlw	.1
	btfsc	STATUS,Z
	goto	NoSumaMas
	
	movf	Dias2daF,W
	addwf	SumaTotalMinutos,F
	
	movf	CantidadDeEscalones,W
	xorlw	.2
	btfsc	STATUS,Z
	goto	NoSumaMas
	
	movf	DiasMadur,W
	addwf	SumaTotalMinutos,F
	
	movf	CantidadDeEscalones,W
	xorlw	.3
	btfsc	STATUS,Z
	goto	NoSumaMas
	
	movf	MinutosE4,W
	addwf	SumaTotalMinutos,F
	
NoSumaMas
	bcf	PCLATH,3
	
	movf	ContadorMinutos,W
	subwf	SumaTotalMinutos,W
	
	call	EscribirEntero
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Mins
	call	LCD_MensajeAlto
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	ModoFuncionamientoMacerado
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	Info0Macerado
	
	bcf	PCLATH,3
	goto	FinInt

ModoFuncionamientoMacerado
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Modo
	call	LCD_Mensaje
	
	movlw	':'
	call	LCD_EnviaDato
	
	call	LCD_Linea2
	
	movlw	TError
	
	btfsc	ModoApagado
	movlw	Apagado
	
	btfsc	ModoFermentar
	movlw	Macerar
	
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	AjustarToleranciaMacerado
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	Info1Macerado
	
	bcf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinInt
	
	movf	ModoDeFuncionamiento,W
	movwf	ModoDeFuncionamientoTemporario

CambiarModoDeFuncionamientoMacerado
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Modo
	call	LCD_Mensaje
	
	movlw	':'
	call	LCD_EnviaDato
	
	call	LCD_Linea2
	
	movlw	TError
	
	btfsc	ModoDeFuncionamientoTemporario,1
	movlw	Macerar
	
	btfsc	ModoDeFuncionamientoTemporario,2
	movlw	Apagado
	
	call	LCD_Mensaje
	
	movlw	.1
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	btfsc	STATUS,Z
	goto	ModoFuncionamientoMacerado
	
	bcf	STATUS,C
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	rrf	ModoDeFuncionamientoTemporario,F
	
	btfsc	ModoDeFuncionamientoTemporario,0
	bsf	ModoDeFuncionamientoTemporario,2
	
	bcf	ModoDeFuncionamientoTemporario,0
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	rlf	ModoDeFuncionamientoTemporario,F
	
	btfsc	ModoDeFuncionamientoTemporario,3
	bsf	ModoDeFuncionamientoTemporario,1
	
	bcf	ModoDeFuncionamientoTemporario,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	CambiarModoDeFuncionamientoMacerado
	
	movf	ModoDeFuncionamientoTemporario,W
	xorwf	ModoDeFuncionamiento,W
	
	btfsc	STATUS,Z
	goto	ModoFuncionamientoMacerado
	
	btfsc	ModoFermentar
	goto	ConfirmarSalirModoMacerar
	
	btfsc	ModoDeFuncionamientoTemporario,1
	goto	ConfirmarModoMacerar
	
	bcf	PCLATH,3
	
	movf	ModoDeFuncionamientoTemporario,W
	movwf	ModoDeFuncionamiento
	
	Bank1
	movlw	0x08
	movwf	EEADR
	Bank0
	movf	ModoDeFuncionamiento,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	goto	FinInt

ConfirmarSalirModoMacerar
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Detener
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	Macerado
	call	LCD_MensajeAlto
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	ModoFuncionamientoMacerado
	
	bcf	PCLATH,3
	
	movf	ModoDeFuncionamientoTemporario,W
	movwf	ModoDeFuncionamiento
	
	Bank1
	movlw	0x08
	movwf	EEADR
	Bank0
	movf	ModoDeFuncionamiento,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	goto	FinInt

ConfirmarModoMacerar
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Iniciar
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	Macerado
	call	LCD_MensajeAlto
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	ModoFuncionamientoMacerado
	
	bcf	PCLATH,3
	
	clrf	ContadorMinutos
	clrf	Contador4SegundosParaMinutos
	
	movf	ModoDeFuncionamientoTemporario,W
	movwf	ModoDeFuncionamiento
	
	Bank1
	movlw	0x08
	movwf	EEADR
	Bank0
	movf	ModoDeFuncionamiento,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x19
	movwf	EEADR
	Bank0
	movf	ContadorMinutos,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	goto	FinInt

AjustarToleranciaMacerado
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Tolerancia
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	.177
	;movlw	b'00010000'		; +/-
	call	LCD_EnviaDato
	
	movf	DeltaTMSB,W
	movwf	TemperaturaAEscribirEntero
	movf	DeltaTLSB,W
	movwf	TemperaturaAEscribirDecimal
	call	EscribirTemperatura2Decimales
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Grados
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,W
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	PreguntaConfigurarProgramaMacerado
	
	movf	Botones,W
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	ModoFuncionamientoMacerado
	
	bcf	PCLATH,3
	
	movf	Botones,W
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinInt
	
	movf	DeltaTLSB,W
	movwf	TemporarioLSB
	movf	DeltaTMSB,W
	movwf	TemporarioMSB

CambiarToleranciaMacerado
	bcf	PCLATH,3
	
	call	LCD_Linea2
	
	movlw	.177
	;movlw	b'00010000'		; +/-
	call	LCD_EnviaDato
	
	movf	TemporarioMSB,W
	movwf	TemperaturaAEscribirEntero
	movf	TemporarioLSB,W
	movwf	TemperaturaAEscribirDecimal
	call	EscribirTemperatura2Decimales
	
	movlw	.1			; La flechita
	call	LCD_EnviaDato
	
	movlw	Grados
	call	LCD_Mensaje
	
	movlw	' '
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenLosBotonesOPasenUnosMs
	call	EsperarBotonesVariosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,W
	btfsc	STATUS,Z
	goto	AjustarToleranciaMacerado
	
	bcf	PCLATH,3
	
	movf	Botones,W
	xorlw	b'00000001'
	btfsc	STATUS,Z
	call	IncrementarX4Temporario
	
	movf	Botones,W
	xorlw	b'00000010'
	btfsc	STATUS,Z
	call	DecrementarX4Temporario
	
	movf	TemporarioMSB,W
	xorlw	.5
	btfsc	STATUS,Z
	clrf	TemporarioLSB
	
	movf	TemporarioMSB,W
	btfsc	STATUS,Z
	call	CortarEn025
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	CambiarToleranciaMacerado
	
	bcf	PCLATH,3
	
	Bank1
	movlw	0x09
	movwf	EEADR
	Bank0
	movf	TemporarioLSB,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x0A
	movwf	EEADR
	Bank0
	movf	TemporarioMSB,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	bsf	PCLATH,3
	
	movf	TemporarioLSB,W
	movwf	DeltaTLSB
	movf	TemporarioMSB,W
	movwf	DeltaTMSB
	goto	AjustarToleranciaMacerado

PreguntaConfigurarProgramaMacerado
	bcf	PCLATH,3
	call	LCD_Borra
	
	movlw	Configurar
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	Programa
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	Info0Macerado
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	AjustarToleranciaMacerado
	
	bcf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	FinInt
	
	movf	CantidadDeEscalones,0
	movwf	CantidadDeEscalonesTemporario
	
ConfigurarProgramaMacerado
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	CantidadDe
	call	LCD_MensajeAlto
	
	call	LCD_Linea2
	
	movlw	Escalones
	call	LCD_MensajeAlto
	
	movf	CantidadDeEscalonesTemporario,0
	call	EscribirEntero
	
	movlw	.1
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenLosBotonesOPasenUnosMs
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	incf	CantidadDeEscalonesTemporario,F
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	decf	CantidadDeEscalonesTemporario,F
	
	movf	CantidadDeEscalonesTemporario,W
	xorlw	.5
	btfsc	STATUS,Z
	decf	CantidadDeEscalonesTemporario,F
	
	movf	CantidadDeEscalonesTemporario,W
	;xorlw	.0
	btfsc	STATUS,Z
	incf	CantidadDeEscalonesTemporario,F
	
	movf	Botones,0
	btfsc	STATUS,Z
	goto	FinInt
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	ConfigurarProgramaMacerado
	
	movf	CantidadDeEscalonesTemporario,W
	movwf	CantidadDeEscalones
	
	movlw	.1
	movwf	EscalonAjustando
	
ConfigurarEscalones
	incf	CantidadDeEscalones,W
	subwf	EscalonAjustando,W
	btfsc	STATUS,Z
	goto	GuardarAjusteMacerado
	
	movf	EscalonAjustando,W
	btfsc	STATUS,Z
	goto	CancelarAjusteMacerado
	
	movlw	.1
	subwf	EscalonAjustando,W
	btfsc	STATUS,Z
	call	CargarDatosPrimerEscalon
	
	movlw	.2
	subwf	EscalonAjustando,W
	btfsc	STATUS,Z
	call	CargarDatosSegundoEscalon
	
	movlw	.3
	subwf	EscalonAjustando,W
	btfsc	STATUS,Z
	call	CargarDatosTercerEscalon
	
	movlw	.4
	subwf	EscalonAjustando,W
	btfsc	STATUS,Z
	call	CargarDatosCuartoEscalon
	
	bcf	PCLATH,3
	call	LCD_Borra
	
	movf	Texto1,W
	call	LCD_MensajeAlto
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Escalon
	call	LCD_MensajeAlto
	
	call	LCD_Linea2
	
	movf	Texto2,W
	call	EscribirEntero
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	Mins
	call	LCD_MensajeAlto
	
	movlw	.15
	call	LCD_PosicionLinea2
	
	movlw	0x04			; Dere a Izda
	call	LCD_EnviaComando
	
	movlw	GradosAlReves
	call	LCD_Mensaje
	
	movlw	' '
	call	LCD_EnviaDato
	
	movf	TemporarioLSB,W
	movwf	TemperaturaAEscribirDecimal
	movf	TemporarioMSB,W
	movwf	TemperaturaAEscribirEntero
	
	call	EscribirTemperaturaDeAtrasParaAdelante
	
	movlw	0x06			; Izda a Dere
	call	LCD_EnviaComando
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	goto	FinIntRestaurandoCambios
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	incf	EscalonAjustando,F
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	decf	EscalonAjustando,F
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfsc	STATUS,Z
	call	AjustarEscalon
	
	bsf	PCLATH,3
	goto	ConfigurarEscalones

CargarDatosPrimerEscalon
	movlw	T1er
	movwf	Texto1
	
	movf	Dias1raF,W
	movwf	Texto2
	
	movf	Temp1raFLSB,W
	movwf	TemporarioLSB
	movf	Temp1raFMSB,W
	movwf	TemporarioMSB
	return

CargarDatosSegundoEscalon
	movlw	T2do
	movwf	Texto1
	
	movf	Dias2daF,W
	movwf	Texto2
	
	movf	Temp2daFLSB,W
	movwf	TemporarioLSB
	movf	Temp2daFMSB,W
	movwf	TemporarioMSB
	return

CargarDatosTercerEscalon
	movlw	T3er
	movwf	Texto1
	
	movf	DiasMadur,W
	movwf	Texto2
	
	movf	TempMaduracionLSB,W
	movwf	TemporarioLSB
	movf	TempMaduracionMSB,W
	movwf	TemporarioMSB
	return

CargarDatosCuartoEscalon
	movlw	T4to
	movwf	Texto1
	
	movf	MinutosE4,W
	movwf	Texto2
	
	movf	TempE4LSB,W
	movwf	TemporarioLSB
	movf	TempE4MSB,W
	movwf	TemporarioMSB
	return

AjustarEscalon
	bcf	PCLATH,3
	
	call	LCD_Linea2
	
	movf	Texto2,W
	call	EscribirEntero
	
	movlw	.1
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenLosBotonesOPasenUnosMs
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	return
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	incf	Texto2,F
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	decf	Texto2,F
	
	movf	Texto2,0
	xorlw	.61
	btfsc	STATUS,Z
	decf	Texto2,F
	
	movf	Texto2,0
	xorlw	.4
	btfsc	STATUS,Z
	incf	Texto2,F
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	AjustarEscalon
	
	bcf	PCLATH,3
	
	call	LCD_Linea2
	
	movf	Texto2,W
	call	EscribirEntero
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	0x04			; Dere a Izda
	call	LCD_EnviaComando
	
AjustarActualSegundoValor
	bcf	PCLATH,3
	
	movlw	.15
	call	LCD_PosicionLinea2
	
	movlw	GradosAlReves
	call	LCD_Mensaje
	
	movlw	.1
	call	LCD_EnviaDato
	
	movf	TemporarioLSB,W	
	movwf	TemperaturaAEscribirDecimal
	movf	TemporarioMSB,W
	movwf	TemperaturaAEscribirEntero
	call	EscribirTemperaturaDeAtrasParaAdelante
	
	movlw	' '
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenLosBotonesOPasenUnosMs
	call	EsperarBotonesVariosSegundos
	
	movf	Botones,0
	btfsc	STATUS,Z
	return
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	call	IncrementarX8Temporario
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	call	DecrementarX8Temporario
	
	movf	TemporarioMSB,0
	xorlw	.120
	btfsc	STATUS,Z
	clrf	TemporarioLSB
	
	movf	TemporarioMSB,0
	xorlw	b'00011101'
	btfsc	STATUS,Z
	call	IncrementarX8Temporario
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	goto	AjustarActualSegundoValor
	
	movlw	.1
	subwf	EscalonAjustando,W
	btfsc	STATUS,Z
	call	GuardarPrimerEscalon
	
	movlw	.2
	subwf	EscalonAjustando,W
	btfsc	STATUS,Z
	call	GuardarSegundoEscalon
	
	movlw	.3
	subwf	EscalonAjustando,W
	btfsc	STATUS,Z
	call	GuardarTercerEscalon
	
	movlw	.4
	subwf	EscalonAjustando,W
	btfsc	STATUS,Z
	call	GuardarCuartoEscalon
	return

GuardarPrimerEscalon
	movf	Texto2,W
	movwf	Dias1raF
	
	movf	TemporarioLSB,W
	movwf	Temp1raFLSB
	movf	TemporarioMSB,W
	movwf	Temp1raFMSB
	return

GuardarSegundoEscalon
	movf	Texto2,W
	movwf	Dias2daF
	
	movf	TemporarioLSB,W
	movwf	Temp2daFLSB
	movf	TemporarioMSB,W
	movwf	Temp2daFMSB
	return

GuardarTercerEscalon
	movf	Texto2,W
	movwf	DiasMadur
	
	movf	TemporarioLSB,W
	movwf	TempMaduracionLSB
	movf	TemporarioMSB,W
	movwf	TempMaduracionMSB
	return

GuardarCuartoEscalon
	movf	Texto2,W
	movwf	MinutosE4
	
	movf	TemporarioLSB,W
	movwf	TempE4LSB
	movf	TemporarioMSB,W
	movwf	TempE4MSB
	return

GuardarAjusteMacerado
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Guardar
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	LosAjustes
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	CancelarAjusteMacerado
	
	movf	CantidadDeEscalones,W
	movwf	EscalonAjustando
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	ConfigurarEscalones
	
	call	MandarConfiguracionAEEPROM
	goto	FinInt

CancelarAjusteMacerado
	bcf	PCLATH,3
	
	call	LCD_Borra
	
	movlw	Cancelar
	call	LCD_Mensaje
	
	call	LCD_Linea2
	
	movlw	LosAjustes
	call	LCD_Mensaje
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesVariosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	GuardarAjusteMacerado
	
	movlw	.1
	movwf	EscalonAjustando
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	ConfigurarEscalones
	
	bcf	PCLATH,3
	goto	FinIntRestaurandoCambios

;-------------------------------------------------------------------------------
;	Subrutinas memoria alta
;-------------------------------------------------------------------------------
EscribirSegundoRenglonMacerando
	bcf	PCLATH,3
	
	call	LCD_Linea2
	
	movf	Texto1,W
	call	LCD_EnviaDato
	
	movlw	')'
	call	LCD_EnviaDato
	
	movlw	.15
	call	LCD_PosicionLinea2
	
	movlw	0x04			; Dere a Izda
	call	LCD_EnviaComando
	
	movlw	Snim
	call	LCD_MensajeAlto
	
	movf	Texto3,W
	call	ConvertirABCD
	
	movf	Unidades,W
	call	LCD_EnviaNumero
	
	movf	Decenas,W
	btfss	STATUS,Z
	call	LCD_EnviaNumero
	
	movlw	Ed
	call	LCD_MensajeAlto
	
	movf	Texto2,W
	call	ConvertirABCD
	
	movf	Unidades,W
	call	LCD_EnviaNumero
	
	movf	Decenas,W
	btfss	STATUS,Z
	call	LCD_EnviaNumero
	
	movlw	' '
	call	LCD_EnviaDato
	
	movlw	0x06			; Izda a Dere
	call	LCD_EnviaComando
	return

Retardo_1s
	clrwdt
	bcf	INTCON,7
	movlw	D'6'
	movwf	CounterC
	movlw	D'19'
	movwf	CounterB
	movlw	D'173'
	movwf	CounterA
	decfsz	CounterA,1
	goto	$-1
	decfsz	CounterB,1
	goto	$-3
	decfsz	CounterC,1
	goto	$-5
	bsf	INTCON,7
	retlw	0

Retardo_500ms
	clrwdt
	bcf	INTCON,7
	movlw	D'3'
	movwf	CounterC
	movlw	D'138'
	movwf	CounterB
	movlw	D'86'
	movwf	CounterA
	decfsz	CounterA,1
	goto	$-1
	decfsz	CounterB,1
	goto	$-3
	decfsz	CounterC,1
	goto	$-5
	bsf	INTCON,7
	retlw	0

MandarConfiguracionAEEPROM
	bcf	PCLATH,3
	
	Bank1
	movlw	0x00
	movwf	EEADR
	Bank0
	movf	Flags,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x01
	movwf	EEADR
	Bank0
	movf	Flags1,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x02
	movwf	EEADR
	Bank0
	movf	Dias1raF,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x03
	movwf	EEADR
	Bank0
	movf	Dias2daF,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x04
	movwf	EEADR
	Bank0
	movf	DiasMadur,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x08
	movwf	EEADR
	Bank0
	movf	ModoDeFuncionamiento,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x09
	movwf	EEADR
	Bank0
	movf	DeltaTLSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x0A
	movwf	EEADR
	Bank0
	movf	DeltaTMSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x0B
	movwf	EEADR
	Bank0
	movf	TempModoTermostatoLSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x0C
	movwf	EEADR
	Bank0
	movf	TempModoTermostatoMSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x0D
	movwf	EEADR
	Bank0
	movf	Temp1raFLSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x0E
	movwf	EEADR
	Bank0
	movf	Temp1raFMSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x0F
	movwf	EEADR
	Bank0
	movf	Temp2daFLSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x10
	movwf	EEADR
	Bank0
	movf	Temp2daFMSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x11
	movwf	EEADR
	Bank0
	movf	TempMaduracionLSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x12
	movwf	EEADR
	Bank0
	movf	TempMaduracionMSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x13
	movwf	EEADR
	Bank0
	movf	TempPasarAModoTermostatoLSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x14
	movwf	EEADR
	Bank0
	movf	TempPasarAModoTermostatoMSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x15
	movwf	EEADR
	Bank0
	movf	CantidadDeEscalones,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x16
	movwf	EEADR
	Bank0
	movf	MinutosE4,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x17
	movwf	EEADR
	Bank0
	movf	TempE4LSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x18
	movwf	EEADR
	Bank0
	movf	TempE4MSB,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	return

RestaurarCambios
	bcf	PCLATH,3
	
	bcf	LuegoTermostato
	movlw	0x00
	call	EEPROM_LeeDato
	andlw	b'01000000'
	btfss	STATUS,Z
	bsf	LuegoTermostato
	call	Retardo_20ms
	
	movlw	0x02
	call	EEPROM_LeeDato
	movwf	Dias1raF
	call	Retardo_20ms
	
	movlw	0x03
	call	EEPROM_LeeDato
	movwf	Dias2daF
	call	Retardo_20ms
	
	movlw	0x04
	call	EEPROM_LeeDato
	movwf	DiasMadur
	call	Retardo_20ms
	
	movlw	0x0D
	call	EEPROM_LeeDato
	movwf	Temp1raFLSB
	call	Retardo_20ms
	
	movlw	0x0E
	call	EEPROM_LeeDato
	movwf	Temp1raFMSB
	call	Retardo_20ms
	
	movlw	0x0F
	call	EEPROM_LeeDato
	movwf	Temp2daFLSB
	call	Retardo_20ms
	
	movlw	0x10
	call	EEPROM_LeeDato
	movwf	Temp2daFMSB
	call	Retardo_20ms
	
	movlw	0x11
	call	EEPROM_LeeDato
	movwf	TempMaduracionLSB
	call	Retardo_20ms
	
	movlw	0x12
	call	EEPROM_LeeDato
	movwf	TempMaduracionMSB
	call	Retardo_20ms
	
	movlw	0x13
	call	EEPROM_LeeDato
	movwf	TempPasarAModoTermostatoLSB
	call	Retardo_20ms
	
	movlw	0x14
	call	EEPROM_LeeDato
	movwf	TempPasarAModoTermostatoMSB
	call	Retardo_20ms
	
	movlw	0x15
	call	EEPROM_LeeDato
	movwf	CantidadDeEscalones
	call	Retardo_20ms
	
	movlw	0x16
	call	EEPROM_LeeDato
	movwf	MinutosE4
	call	Retardo_20ms
	
	movlw	0x17
	call	EEPROM_LeeDato
	movwf	TempE4LSB
	call	Retardo_20ms
	
	movlw	0x18
	call	EEPROM_LeeDato
	movwf	TempE4MSB
	call	Retardo_20ms
	
	return

VerEnQueFaseEsta
	bcf	Fermentando1
	bcf	Fermentando2
	bcf	Madurando
	
	movf	Dias1raF,W
	movwf	SumaDiasFases
	
	subwf	ContadorDias,W
	btfsc	STATUS,C
	goto	PasoLa1raF
	
	bcf	PCLATH,3
	
	bsf	Fermentando1
	
	movf	Temp1raFLSB,W
	movwf	TemperaturaBuscadaLSB
	movf	Temp1raFMSB,W
	movwf	TemperaturaBuscadaMSB
	
	movlw	Primera
	call	LCD_Mensaje
	movlw	' '
	call	LCD_EnviaDato
	movlw	Fermentacion
	call	LCD_Mensaje
	return

PasoLa1raF
	movf	Dias1raF,W
	addwf	Dias2daF,W
	
	movwf	SumaDiasFases
	
	subwf	ContadorDias,W
	btfsc	STATUS,C
	goto	PasoLa2daF
	
	bcf	PCLATH,3
	
	bsf	Fermentando2
	
	movf	Temp2daFLSB,W
	movwf	TemperaturaBuscadaLSB
	movf	Temp2daFMSB,W
	movwf	TemperaturaBuscadaMSB
	
	movlw	Segunda
	call	LCD_Mensaje
	movlw	' '
	call	LCD_EnviaDato
	movlw	Fermentacion
	call	LCD_Mensaje
	return

PasoLa2daF
	movf	Dias1raF,W
	addwf	Dias2daF,W
	addwf	DiasMadur,W
	
	movwf	SumaDiasFases
	
	subwf	ContadorDias,W
	btfsc	STATUS,C
	goto	PasoLaMadur
	
	bcf	PCLATH,3
	
	bsf	Madurando
	
	movf	TempMaduracionLSB,W
	movwf	TemperaturaBuscadaLSB
	movf	TempMaduracionMSB,W
	movwf	TemperaturaBuscadaMSB
	
	movlw	Maduracion
	call	LCD_Mensaje
	return

PasoLaMadur
	bsf	ProcesoFinalizado
	
	bcf	PCLATH,3
	
	Bank1
	movlw	0x00
	movwf	EEADR
	Bank0
	movf	Flags,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	movlw	FinDelProceso
	call	LCD_Mensaje
	
	btfss	LuegoTermostato
	return
	
	bcf	ProcesoFinalizado
	
	movf	TempPasarAModoTermostatoLSB,W
	movwf	TempModoTermostatoLSB
	movwf	TemperaturaBuscadaLSB
	
	movf	TempPasarAModoTermostatoMSB,W
	movwf	TempModoTermostatoMSB
	movwf	TemperaturaBuscadaMSB
	
	clrf	ModoDeFuncionamiento
	bsf	ModoTermostato
	
	Bank1
	movlw	0x00
	movwf	EEADR
	Bank0
	movf	Flags,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	return

Comparar16Bits
	movf	AComparar_1MSB,W
	xorlw	b'10000000'
	movwf	CompararTemporario
	
	movf	AComparar_2MSB,W
	xorlw	b'10000000'
	subwf	CompararTemporario,W
	
	btfss	STATUS,Z
	return
	
	movf	AComparar_2LSB,W
	subwf	AComparar_1LSB
	return
	
	;	Z = 1	Son iguales
	;	C = 0	AComparar_1 es menor
	;	C = 1	AComparar_1 es mayor (o igual)

VerSiParaOArranca
	btfsc	ModoApagado
	goto	Parar
						;------ CalcularTempSuperiorEInferior
	movf	DeltaTMSB,W
	addwf	TemperaturaBuscadaMSB,W
	movwf	TemperaturaBuscadaSuperiorMSB
	
	movf	DeltaTLSB,W
	addwf	TemperaturaBuscadaLSB,W
	movwf	TemperaturaBuscadaSuperiorLSB
	
	btfsc	STATUS,DC
	incf	TemperaturaBuscadaSuperiorMSB,F
	
	movlw	b'00001111'
	andwf	TemperaturaBuscadaSuperiorLSB,F
	
	;C = 0 => Resultado negativo
	
	movf	DeltaTMSB,W
	subwf	TemperaturaBuscadaMSB,W
	movwf	TemperaturaBuscadaInferiorMSB
	
	movf	DeltaTLSB,W
	subwf	TemperaturaBuscadaLSB,W
	movwf	TemperaturaBuscadaInferiorLSB
	
	btfss	STATUS,C
	decf	TemperaturaBuscadaInferiorMSB
	
	movlw	b'00001111'
	andwf	TemperaturaBuscadaInferiorLSB,F
						; --- "Le suma uno al límite inferior"
	movlw	.1
	addwf	TemperaturaBuscadaInferiorLSB
	
	btfsc	STATUS,DC
	incf	TemporarioMSB,F
	
	movlw	b'00001111'
	andwf	TemperaturaBuscadaInferiorLSB,F
						; ---
						;------
	
	movf	TemperaturaDecimal,W
	movwf	AComparar_1LSB
	movf	TemperaturaEntero,W
	movwf	AComparar_1MSB
	
	movf	TemperaturaBuscadaSuperiorLSB,W
	movwf	AComparar_2LSB
	movf	TemperaturaBuscadaSuperiorMSB,W
	movwf	AComparar_2MSB
	
	call	Comparar16Bits
	btfsc	STATUS,C
	goto	TMayorOIgualATBS	;Temperatura es mayor o igual a TemperaturaBuscadaSuperior
	
	movf	TemperaturaDecimal,W
	movwf	AComparar_1LSB
	movf	TemperaturaEntero,W
	movwf	AComparar_1MSB
	
	movf	TemperaturaBuscadaInferiorLSB,W
	movwf	AComparar_2LSB
	movf	TemperaturaBuscadaInferiorMSB,W
	movwf	AComparar_2MSB
	
	call	Comparar16Bits
	btfsc	STATUS,C
	return			;Temperatura es mayor o igual a TemperaturaBuscadaInferior
	
	btfss	SalidaParaCalentar
	goto	Parar
;	goto	Arrancar
Arrancar
	btfsc	ActivarSalida
	return
	
	bsf	ActivarSalida
	call	Retardo_500ms
	return

TMayorOIgualATBS
	btfss	SalidaParaCalentar
	goto	Arrancar
;	goto	Parar
Parar
	btfss	ActivarSalida
	return
	
	bcf	ActivarSalida
	call	Retardo_500ms
	return

ResetGeneral
	bcf	PCLATH,3
	
	movlw	Firmware
	call	LCD_MensajeAlto
	
	call	LCD_Linea2
	
	movlw	Reset
	call	LCD_MensajeAlto
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	call	LCD_Borra
	
	movf	Botones,0
	xorlw	b'00000011'
	btfss	STATUS,Z
	return
	
	clrf	Flags
	clrf	Flags1
	
PreguntaTipoSensor
	movlw	b'00000010'
	xorwf	Flags,F
	
	bcf	PCLATH,3
	call	LCD_Linea1
	
	movlw	'A'
	btfsc	ModoSensorTipoB
	movlw	'B'
	
	call	LCD_EnviaDato
	
	call	EsperarQueSueltenBotones
	call	EsperarBotonesUnosSegundos
	
	bsf	PCLATH,3
	
	movf	Botones,0
	xorlw	b'00000001'
	btfsc	STATUS,Z
	goto	PreguntaTipoSensor
	
	movf	Botones,0
	xorlw	b'00000010'
	btfsc	STATUS,Z
	goto	PreguntaTipoSensor

; PreguntaFrioOCalor
	; movlw	b'00000001'
	; xorwf	Flags1,F
	; 
	; bcf	PCLATH,3
	; call	LCD_Linea1
	; 
	; movlw	'F'
	; btfsc	ControlDeMacerado
	; movlw	'C'
	; 
	; call	LCD_EnviaDato
	; 
	; call	EsperarQueSueltenBotones
	; call	EsperarBotonesUnosSegundos
	; 
	; bsf	PCLATH,3
	; 
	; movf	Botones,0
	; xorlw	b'00000001'
	; btfsc	STATUS,Z
	; goto	PreguntaFrioOCalor
	; 
	; movf	Botones,0
	; xorlw	b'00000010'
	; btfsc	STATUS,Z
	; goto	PreguntaFrioOCalor
	
; DisplayPuto
	; bcf	PCLATH,3
	; call	LCD_Borra
	; 
	; movf	Texto1,W
	; call	LCD_EnviaDato
	; 
	; movlw	' '
	; call	LCD_EnviaDato
	; 
	; movf	Texto1,W
	; call	EscribirEntero
	; 
	; call	EsperarQueSueltenBotones
	; call	EsperarBotonesUnosSegundos
	; 
	; bsf	PCLATH,3
	; 
	; movf	Botones,0
	; xorlw	b'00000001'
	; btfsc	STATUS,Z
	; incf	Texto1,F
	; 
	; movf	Botones,0
	; xorlw	b'00000010'
	; btfsc	STATUS,Z
	; decf	Texto1,F
	; 
	; movf	Botones,0
	; btfss	STATUS,Z
	; goto	DisplayPuto
	
	bcf	PCLATH,3
	call	LCD_Borra
	
	movlw	.30
	movwf	Dias1raF
	movwf	Dias2daF
	movwf	DiasMadur
	movwf	MinutosE4
	
	movlw	.2
	movwf	CantidadDeEscalones
	
	movwf	DeltaTMSB
	clrf	DeltaTLSB
	
	movlw	.20
	movwf	TempModoTermostatoMSB
	movwf	TempPasarAModoTermostatoMSB
	
	clrf	TempModoTermostatoLSB
	clrf	TempPasarAModoTermostatoLSB
	
	movlw	.30
	movwf	Temp1raFMSB
	movwf	Temp2daFMSB
	movwf	TempMaduracionMSB
	movwf	TempE4MSB
	
	clrf	Temp1raFLSB
	clrf	Temp2daFLSB
	clrf	TempMaduracionLSB
	clrf	TempE4LSB
	
	clrf	MinutosEncendido
	
	clrf	ContadorParaApagarBacklight
	
	clrf	ModoDeFuncionamiento
	bsf	ModoApagado
	
	clrf	Contador800Segundos
	clrf	ContadorDias
	clrf	ContadorMinutos
	clrf	MinutosEncendido
	clrf	HorasEncendido
	clrf	DiasEncendido
	
	Bank1
	movlw	0x20
	movwf	EEADR
	bcf	STATUS,RP0
	movf	MinutosEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x21
	movwf	EEADR
	bcf	STATUS,RP0
	movf	HorasEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	Bank1
	movlw	0x22
	movwf	EEADR
	bcf	STATUS,RP0
	movf	DiasEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	
	bsf	PCLATH,3			; Ya lo hace arriba
	call	MandarConfiguracionAEEPROM	;Esto es del RG
	return

VerEnQueFaseEstaMacerado
	bcf	Fermentando1
	bcf	Fermentando2
	bcf	Madurando
	bcf	EstaEnElEscalon4
	
	movf	Dias1raF,W
	movwf	SumaDiasFases
	
	subwf	ContadorMinutos,W
	btfsc	STATUS,C
	goto	PasoElPrimerEscalon
	
	bsf	Fermentando1
	
	movf	Temp1raFLSB,W
	movwf	TemperaturaBuscadaLSB
	movf	Temp1raFMSB,W
	movwf	TemperaturaBuscadaMSB
	
	movlw	'1'
	movwf	Texto1
	
	movf	ContadorMinutos,W
	movwf	Texto2
		
	movf	Dias1raF,W
	movwf	Texto3
	
	call	EscribirSegundoRenglonMacerando
	return

PasoElPrimerEscalon
	movf	CantidadDeEscalones,W
	xorlw	.1
	btfsc	STATUS,Z
	goto	PasoElCuartoEscalon
	
	movf	Dias1raF,W
	addwf	Dias2daF,W
	
	movwf	SumaDiasFases
	
	subwf	ContadorMinutos,W
	btfsc	STATUS,C
	goto	PasoElSegundoEscalon
	
	bsf	Fermentando2
	
	movf	Temp2daFLSB,W
	movwf	TemperaturaBuscadaLSB
	movf	Temp2daFMSB,W
	movwf	TemperaturaBuscadaMSB
	
	movlw	'2'
	movwf	Texto1
	
	movf	Dias1raF,W
	subwf	ContadorMinutos,W
	movwf	Texto2
	
	movf	Dias2daF,W
	movwf	Texto3
	
	call	EscribirSegundoRenglonMacerando
	return

PasoElSegundoEscalon
	movf	CantidadDeEscalones,W
	xorlw	.2
	btfsc	STATUS,Z
	goto	PasoElCuartoEscalon
	
	movf	Dias1raF,W
	addwf	Dias2daF,W
	addwf	DiasMadur,W
	
	movwf	SumaDiasFases
	
	subwf	ContadorMinutos,W
	btfsc	STATUS,C
	goto	PasoElTercerEscalon
	
	bsf	Madurando
	
	movf	TempMaduracionLSB,W
	movwf	TemperaturaBuscadaLSB
	movf	TempMaduracionMSB,W
	movwf	TemperaturaBuscadaMSB
	
	movlw	'3'
	movwf	Texto1
	
	movf	Dias1raF,W
	addwf	Dias2daF,W
	subwf	ContadorMinutos,W
	movwf	Texto2
	
	movf	DiasMadur,W
	movwf	Texto3
	
	call	EscribirSegundoRenglonMacerando
	return

PasoElTercerEscalon
	movf	CantidadDeEscalones,W
	xorlw	.3
	btfsc	STATUS,Z
	goto	PasoElCuartoEscalon
	
	movf	Dias1raF,W
	addwf	Dias2daF,W
	addwf	DiasMadur,W
	addwf	MinutosE4,W
	
	movwf	SumaDiasFases
	
	subwf	ContadorMinutos,W
	btfsc	STATUS,C
	goto	PasoElCuartoEscalon
	
	bsf	EstaEnElEscalon4
	
	movf	TempE4LSB,W
	movwf	TemperaturaBuscadaLSB
	movf	TempE4MSB,W
	movwf	TemperaturaBuscadaMSB
	
	movlw	'4'
	movwf	Texto1
	
	movf	Dias1raF,W
	addwf	Dias2daF,W
	addwf	DiasMadur,W
	subwf	ContadorMinutos,W
	movwf	Texto2
	
	movf	MinutosE4,W
	movwf	Texto3
	
	call	EscribirSegundoRenglonMacerando
	return

PasoElCuartoEscalon
	bsf	ProcesoFinalizado
	
	bcf	PCLATH,3
	
	movlw	FinDelProceso
	call	LCD_Mensaje
	
	Bank1
	movlw	0x00
	movwf	EEADR
	Bank0
	movf	Flags,0
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	return

VerSiParaOArrancaMacerando
	btfsc	ModoApagado
	goto	Parar
						;------ CalcularTempSuperiorEInferior
	movf	DeltaTMSB,W
	addwf	TemperaturaBuscadaMSB,W
	movwf	TemperaturaBuscadaSuperiorMSB
	
	movf	DeltaTLSB,W
	addwf	TemperaturaBuscadaLSB,W
	movwf	TemperaturaBuscadaSuperiorLSB
	
	btfsc	STATUS,DC
	incf	TemperaturaBuscadaSuperiorMSB,F
	
	movlw	b'00001111'
	andwf	TemperaturaBuscadaSuperiorLSB,F
	
	;C = 0 => Resultado negativo
	
	movf	DeltaTMSB,W
	subwf	TemperaturaBuscadaMSB,W
	movwf	TemperaturaBuscadaInferiorMSB
	
	movf	DeltaTLSB,W
	subwf	TemperaturaBuscadaLSB,W
	movwf	TemperaturaBuscadaInferiorLSB
	
	btfss	STATUS,C
	decf	TemperaturaBuscadaInferiorMSB
	
	movlw	b'00001111'
	andwf	TemperaturaBuscadaInferiorLSB,F
						; --- "Le suma uno al límite inferior"
	movlw	.1
	addwf	TemperaturaBuscadaInferiorLSB
	
	btfsc	STATUS,DC
	incf	TemporarioMSB,F
	
	movlw	b'00001111'
	andwf	TemperaturaBuscadaInferiorLSB,F
						; ---
						;------
	
	movf	TemperaturaDecimal,W
	movwf	AComparar_1LSB
	movf	TemperaturaEntero,W
	movwf	AComparar_1MSB
	
	movf	TemperaturaBuscadaSuperiorLSB,W
	movwf	AComparar_2LSB
	movf	TemperaturaBuscadaSuperiorMSB,W
	movwf	AComparar_2MSB
	
	call	Comparar16Bits
	btfsc	STATUS,C
	goto	Parar		;Temperatura es mayor o igual a TemperaturaBuscadaSuperior
	
	movf	TemperaturaDecimal,W
	movwf	AComparar_1LSB
	movf	TemperaturaEntero,W
	movwf	AComparar_1MSB
	
	movf	TemperaturaBuscadaInferiorLSB,W
	movwf	AComparar_2LSB
	movf	TemperaturaBuscadaInferiorMSB,W
	movwf	AComparar_2MSB
	
	call	Comparar16Bits
	btfsc	STATUS,C
	return			;Temperatura es mayor o igual a TemperaturaBuscadaInferior
	goto	Arrancar

BeepsMacerando
	btfss	ControlDeMacerado
	return
	
	btfss	ActivarSalida
	return
	
	bsf	Buzzer
	call	Retardo_500ms
	bcf	Buzzer
	
	bsf	Buzzer
	call	Retardo_1s
	bcf	Buzzer
	
	bsf	Buzzer
	call	Retardo_500ms
	bcf	Buzzer
	return

Beep
	bsf	Buzzer
	bcf	PCLATH,3
	call	Retardo_30ms
	bcf	Buzzer
	return

;-------------------------------------------------------------------------------
;	Subrutinas displays 7 segmentos
;-------------------------------------------------------------------------------
Escribir7S
	bcf	PCLATH,3
	bsf	PCLATH,0		;La tabla arranca en 0x100
	
	call	ConvertirA7S
	movwf	NumeroAEscribir7S
	
	movlw	.8
	movwf	CantPulsos7S
	
BucleCantPulsos
	bcf	PCLATH,3
	
	bcf	SieteSeg_CLK
	bcf	SieteSeg_D
	
	btfsc	NumeroAEscribir7S,0
	bsf	SieteSeg_D
	
	call	Retardo_500micros
	bsf	SieteSeg_CLK
	call	Retardo_500micros
	
	rrf	NumeroAEscribir7S,F
	
	bsf	PCLATH,3
	
	decfsz	CantPulsos7S,F
	goto	BucleCantPulsos
	
	bcf	PCLATH,0
	return

EscribirTemp7S
	bcf	PCLATH,3
	
	movf	TemperaturaEntero,W
	movwf	TemperaturaAEscribirEntero
	movf	TemperaturaDecimal,W
	movwf	TemperaturaAEscribirDecimal
	
	bcf	LaTempEsNegativa
	
	btfsc	TemperaturaAEscribirEntero,7
	call	LaTempAEscribirEsNegativa
	
	movf	TemperaturaAEscribirDecimal,W
	
	bsf	PCLATH,0		;La tabla está fuera de 0xFF
	call	TablaCalcularElDecimal
	bcf	PCLATH,0
	
	bsf	PCLATH,3
	call	Escribir7S
	
	movf	TemperaturaAEscribirEntero,W
	bcf	PCLATH,3
	call	ConvertirABCD
	
	movf	Unidades,W
	bsf	PCLATH,3
	call	Escribir7S
	
	movf	Decenas,W
	
	btfsc	STATUS,Z
	movlw	.11
	
	btfsc	LaTempEsNegativa
	movlw	.10
	
	call	Escribir7S
	return

;-------------------------------------------------------------------------------
;	Subrutinas transcurso de los días
;-------------------------------------------------------------------------------
DesbordoElTimer1
	bcf	PIR1,TMR1IF
	
	btfsc	ActivarSalida
	call	Contador4SegundosEncendido
	
	bcf	PCLATH,3
	
	movf	Contador4Segundos,W
	xorlw	.1
	btfsc	STATUS,Z
	call	DS1820ConvertirTemperatura
	
	movf	Contador4Segundos,W
	xorlw	.2
	btfsc	STATUS,Z
	call	DS1820LeerTemperatura
	
	bsf	PCLATH,3
	
	movf	Contador4Segundos,W
	xorlw	.2
	btfsc	STATUS,Z
	call	ProcesarTemperatura
	
	;movf	Contador4Segundos,W
	;xorlw	.3
	;btfsc	STATUS,Z
	;call	EscribirTemp7S
	
	incf	Contador4Segundos,F
	
	movf	Contador4Segundos,W
	xorlw	.5
	btfsc	STATUS,Z
	clrf	Contador4Segundos
	
	bcf	PCLATH,3
	
	btfss	ModoFermentar
	goto	FinInt
	
	bsf	PCLATH,3
	
	movf	Contador4Segundos,W
	xorlw	.1
	btfsc	STATUS,Z
	call	BeepsMacerando
	
	incf	Contador4SegundosParaMinutos,F
	
	movf	Contador4SegundosParaMinutos,W
	xorlw	.15					; ¿Acá no sería 16???, el incf está antes... habría que ver como lo restea...
	btfsc	STATUS,Z
	call	DesbordoElContador4SegundosParaMinutos
	
	bsf	PCLATH,3
	
	incf	Contador4SegundosParaDias,F
	
	movf	Contador4SegundosParaDias,W
	xorlw	.200
	btfsc	STATUS,Z
	call	DesbordoElContador4SegundosParaDias
	
	bcf	PCLATH,3
	goto	FinInt

DesbordoElContador4SegundosParaDias
	btfsc	ControlDeMacerado
	return
	
	clrf	Contador4SegundosParaDias
	incf	Contador800Segundos,F
	
	movf	Contador800Segundos,W
	xorlw	.108
	btfsc	STATUS,Z
	call	DesbordoElContador800Segundos
	
	bcf	PCLATH,3
	
	Bank1
	movlw	0x06
	movwf	EEADR
	bcf	STATUS,RP0
	movf	Contador800Segundos,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	return

DesbordoElContador800Segundos
	clrf	Contador800Segundos
	incf	ContadorDias,F
	
	bcf	PCLATH,3
	
	Bank1
	movlw	0x07
	movwf	EEADR
	bcf	STATUS,RP0
	movf	ContadorDias,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	return

Contador4SegundosEncendido
	incf	CuatroSegundosEncendido,F
	
	movf	CuatroSegundosEncendido,W
	xorlw	.15
	btfss	STATUS,Z
	return
	;call	DesbordoElContador4SegundosEncendido

;DesbordoElContador4SegundosEncendido
	clrf	CuatroSegundosEncendido
	incf	MinutosEncendido,F
	
	movf	MinutosEncendido,W
	xorlw	.60
	btfsc	STATUS,Z
	call	DesbordoElContadorMinutosEncendido
	
	bcf	PCLATH,3
	
	Bank1
	movlw	0x20
	movwf	EEADR
	bcf	STATUS,RP0
	movf	MinutosEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	return

DesbordoElContadorMinutosEncendido
	clrf	MinutosEncendido
	incf	HorasEncendido,F
	
	movf	HorasEncendido,W
	xorlw	.24
	btfsc	STATUS,Z
	call	DesbordoElContadorHorasEncendido
	
	bcf	PCLATH,3
	
	Bank1
	movlw	0x21
	movwf	EEADR
	bcf	STATUS,RP0
	movf	HorasEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	return

DesbordoElContadorHorasEncendido
	clrf	HorasEncendido
	incf	DiasEncendido,F
	
	bcf	PCLATH,3
	
	Bank1
	movlw	0x22
	movwf	EEADR
	bcf	STATUS,RP0
	movf	DiasEncendido,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	return

DesbordoElContador4SegundosParaMinutos
	btfss	ControlDeMacerado
	return
	
	clrf	Contador4SegundosParaMinutos
	
	incf	ContadorMinutos,F
	
	bcf	PCLATH,3
	
	Bank1
	movlw	0x19
	movwf	EEADR
	bcf	STATUS,RP0
	movf	ContadorMinutos,W
	call	EEPROM_EscribeDato
	call	Retardo_20ms
	return

ProcesarTemperatura
	btfss	ModoSensorTipoB
	goto	ProcesarTemperatura1820

ProcesarTemperatura18B20				;Para el DS18B20
	movf	TemperaturaLeidaLSB,W
	andlw	b'00001111'
	movwf	TemperaturaDecimal
	
	swapf	TemperaturaLeidaLSB,W
	andlw	b'00001111'
	movwf	ProcesarTempTemporario
	
	swapf	TemperaturaLeidaMSB,W
	andlw	b'11110000'
	iorwf	ProcesarTempTemporario,W
	movwf	TemperaturaEntero
	return
	
ProcesarTemperatura1820				;Para el DS1820
	movlw	b'00000000'
	
	btfsc	TemperaturaLeidaLSB,0		;Esto es para 'igualar' al B20
	movlw	b'00001000'
	
	movwf	TemperaturaDecimal
	
	bcf	STATUS,C                                                          
	rrf	TemperaturaLeidaLSB,W
	
	btfsc	TemperaturaLeidaMSB,0
	iorlw	b'10000000'
	
	movwf	TemperaturaEntero
	return

end
